import * as React from "react";
import * as PropTypes from "prop-types";
import { FailedBackgroundJobsPanel } from "./components";
import { ReactUtilsType } from "../utils";
import { HasMaybeChildren, Theme } from "../themes/theme";
import { ContextProps } from "../ReactContext";
import Form from "@rjsf/core";
import { FieldProps as RJSFFieldProps, WidgetProps as RJSFWidgetProps, Field, Widget, TemplatesType } from "@rjsf/utils";
import ApiClient, { ApiClientImplementation } from "../ApiClient";
import KeyHandlerService, { ShortcutKeys } from "../services/key-handler-service";
import SettingsService, { Settings } from "../services/settings-service";
import FocusService from "../services/focus-service";
import BlockerService from "../services/blocker-service";
import CustomEventService from "../services/custom-event-service";
import SubmitHookService, { SubmitHook } from "../services/submit-hook-service";
import DOMIdService from "../services/dom-id-service";
import IdService from "../services/id-service";
import RootInstanceService from "../services/root-instance-service";
import SingletonMapService from "../services/singleton-map-service";
export interface LajiFormProps extends HasMaybeChildren {
    apiClient?: ApiClientImplementation;
    lang?: Lang;
    formData?: any;
    schema?: any;
    uiSchema?: any;
    topOffset?: number;
    bottomOffset?: number;
    formContext?: any;
    uiSchemaContext?: any;
    settings?: any;
    id?: string;
    googleApiKey?: string;
    notifier?: Notifier;
    fields?: {
        [name: string]: Field;
    };
    widgets?: {
        [name: string]: Widget;
    };
    templates?: {
        [name: string]: TemplatesType;
    };
    autoFocus?: boolean;
    componentDidMount?: () => void;
    onError?: (e: Error, i: React.ErrorInfo) => void;
    onChange?: (formData: any) => void;
    optimizeOnChange?: boolean;
    showShortcutButton?: boolean;
    renderSubmit?: boolean;
    submitText?: string;
    onSubmit?: (data: {
        formData: any;
    }) => void;
    onValidationError?: (extraErrors: any) => void;
    validators?: any;
    warnings?: any;
    onSettingsChange?: (settings: any, global: boolean) => void;
    mediaMetadata?: MediaMetadata;
    theme?: Theme;
    lajiGeoServerAddress?: string;
}
export interface LajiFormState {
    submitHooks?: SubmitHook[];
    formContext: FormContext;
    formData?: any;
    extraErrors?: any;
    error?: boolean;
    runningSubmitHooks?: boolean;
}
export interface MediaMetadata {
    capturerVerbatim: string;
    intellectualOwner: string;
    intellectualRights: string;
}
export interface FormContext {
    translations: ByLang;
    lang: Lang;
    uiSchemaContext: any;
    formRef: React.RefObject<Form>;
    topOffset: number;
    bottomOffset: number;
    formID: string;
    googleApiKey: string;
    notifier: Notifier;
    apiClient: ApiClient;
    Label: React.Component;
    formDataTransformers?: any[];
    _parentLajiFormId?: number;
    mediaMetadata?: MediaMetadata;
    contextId: number;
    theme: Theme;
    setTimeout: (fn: () => void, time?: number) => void;
    utils: ReactUtilsType;
    lajiGeoServerAddress: string;
    globals: Record<string, unknown>;
    services: {
        keyHandler: KeyHandlerService;
        settings: SettingsService;
        focus: FocusService;
        blocker: BlockerService;
        customEvents: CustomEventService;
        submitHooks: SubmitHookService;
        DOMIds: DOMIdService;
        ids: IdService;
        rootInstance: RootInstanceService;
        singletonMap: SingletonMapService;
    };
}
export declare type Lang = "fi" | "en" | "sv";
export interface FieldProps extends RJSFFieldProps<any, any, FormContext> {
    uiSchema: any;
    errorSchema: any;
    formContext: FormContext;
}
export interface WidgetProps extends RJSFWidgetProps<any, any, FormContext> {
    uiSchema: any;
    errorSchema: any;
    formContext: FormContext;
}
export declare type NotifyMessager = (msg: string) => void;
export interface Notifier {
    success: NotifyMessager;
    info: NotifyMessager;
    warning: NotifyMessager;
    error: NotifyMessager;
}
export declare type ByLang = {
    [key: string]: string;
};
export declare type Translations = Record<Lang, ByLang>;
export default class LajiForm extends React.Component<LajiFormProps, LajiFormState> {
    static contextType: React.Context<ContextProps>;
    private contextMemoizeKey;
    private contextFormMemoizeKey;
    memoizedContext: ContextProps;
    memoizedFormContext: FormContext;
    translations: Translations;
    bgJobRef: React.RefObject<FailedBackgroundJobsPanel>;
    shortcutHelpRef: React.RefObject<any>;
    apiClient: ApiClient;
    _id: number;
    propagateSubmit: boolean;
    formRef: React.RefObject<Form<any, import("@rjsf/utils").RJSFSchema, any>>;
    mounted: boolean;
    keyCombo: string;
    defaultNotifier: Notifier;
    validating: boolean;
    cachedNonliveValidations: any;
    helpVisible: boolean;
    helpTimeout: number;
    helpStarted: boolean;
    eventListeners: [typeof document | typeof window, string, (e: Event) => void][];
    timeouts: number[];
    static propTypes: {
        uiSchemaContext: PropTypes.Requireable<object>;
        settings: PropTypes.Requireable<object>;
        validators: PropTypes.Requireable<object>;
        warnings: PropTypes.Requireable<object>;
    };
    static defaultProps: {
        lang: Lang;
        schema: {};
        uiSchema: {};
    };
    constructor(props: LajiFormProps);
    UNSAFE_componentWillReceiveProps(props: LajiFormProps): void;
    getStateFromProps(props: LajiFormProps): LajiFormState;
    getMemoizedFormContext(props: LajiFormProps): FormContext;
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentDidCatch(e: Error, i: React.ErrorInfo): void;
    componentDidUpdate(prevProps: LajiFormProps): void;
    onSubmitHooksChange: (submitHooks: SubmitHook[], callback?: (() => void) | undefined) => void;
    resetShortcuts(shortcuts?: ShortcutKeys): void;
    constructTranslations(): Translations;
    getDefaultNotifier: () => Notifier;
    onChange: ({ formData }: {
        formData: any;
    }) => void;
    getFields: (_fields?: {
        [name: string]: Field<any, import("@rjsf/utils").RJSFSchema, any>;
    } | undefined) => {
        [x: string]: Field<any, import("@rjsf/utils").RJSFSchema, any>;
    };
    getWidgets: (_widgets?: {
        [name: string]: Widget<any, import("@rjsf/utils").RJSFSchema, any>;
    } | undefined) => {
        [x: string]: Widget<any, import("@rjsf/utils").RJSFSchema, any>;
    };
    getTemplates: (_templates?: {
        [name: string]: TemplatesType<any, import("@rjsf/utils").RJSFSchema, any>;
    } | undefined) => {
        [x: string]: TemplatesType<any, import("@rjsf/utils").RJSFSchema, any>;
    };
    getContext: (props: LajiFormProps, context: ContextProps) => ContextProps;
    render(): JSX.Element | null;
    renderSubmitHooks: () => JSX.Element | null | undefined;
    validateAndSubmit: (warnings?: boolean, onlySchema?: boolean) => Promise<void>;
    validate: (warnings?: boolean, nonlive?: boolean, onlySchema?: boolean) => Promise<unknown>;
    onSubmit: (onlySchemaValidations?: "onlySchemaValidations" | undefined) => false | undefined;
    popErrorListIfNeeded: () => void;
    _onDefaultSubmit: (e: React.SyntheticEvent) => void;
    submit: () => void;
    submitOnlySchemaValidations: () => void;
    getShorcutButtonTooltip: () => string | undefined;
    toggleHelp: (e: React.MouseEvent) => void;
    showHelp: () => void;
    dismissHelp: (e: Event | React.SyntheticEvent) => void;
    keyFunctions: {
        navigate: (e: KeyboardEvent, { reverse }: any) => void;
        help: (e: KeyboardEvent, { delay }: any) => boolean;
        revalidate: () => void;
    };
    pushBlockingLoader: () => void;
    popBlockingLoader: () => void;
    getSettings(global?: boolean): {};
    onSettingsChange: (settings: Settings, global?: boolean) => void;
    addEventListener: (target: typeof document | typeof window, name: string, fn: (e: Event) => void) => void;
    setTimeout: (fn: () => void, time?: number) => number;
    destroy: () => void;
}
