"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArrayFieldPatched = exports.onArrayFieldChange = exports.copyItemFunction = void 0;
const React = require("react");
const merge = require("deepmerge");
const utils_1 = require("../../utils");
const BaseComponent_1 = require("../BaseComponent");
const ArrayFieldTemplate_1 = require("../templates/ArrayFieldTemplate");
const ReactContext_1 = require("../../ReactContext");
const core_1 = require("@rjsf/core");
// Doesn't work with arrays properly since uses JSON Pointers but not JSON path.
// e.g. "copy all array item values expect these" is impossible.
const copyItemFunction = (that, copyItem) => (props, { type, filter }) => {
    const { schema, registry, formContext } = that.props;
    const defaultItem = utils_1.getDefaultFormState(schema.items, undefined);
    copyItem = formContext.utils.filterItemIdsDeeply(copyItem, that.props.idSchema.$id);
    const source = type === "blacklist" ? defaultItem : copyItem;
    const filtered = filter.sort().reverse().reduce((target, f) => {
        let sourceValue;
        try {
            sourceValue = utils_1.parseJSONPointer(source, f);
        }
        catch (e) {
            const schema = utils_1.schemaJSONPointer(schema.items, f);
            sourceValue = utils_1.getDefaultFormState(schema, undefined);
        }
        return utils_1.updateFormDataWithJSONPointer({ schema: schema.items, formData: target, registry }, sourceValue, f);
    }, type === "blacklist" ? copyItem : defaultItem);
    const tmpIdTree = that.props.formContext.services.ids.getRelativeTmpIdTree(props.idSchema.$id);
    return utils_1.addLajiFormIds(filtered, tmpIdTree, false)[0];
};
exports.copyItemFunction = copyItemFunction;
function onArrayFieldChange(formData, props) {
    const tmpIdTree = props.formContext.services.ids.getRelativeTmpIdTree(props.idSchema.$id);
    return utils_1.addLajiFormIds(formData, tmpIdTree, false)[0];
}
exports.onArrayFieldChange = onArrayFieldChange;
const { ArrayField } = core_1.getDefaultRegistry().fields;
class ArrayFieldPatched extends ArrayField {
    constructor(...params) {
        super(...params);
        const { _getNewFormDataRow } = this;
        this._getNewFormDataRow = () => {
            const tmpIdTree = this.props.formContext.services.ids.getRelativeTmpIdTree(this.props.idSchema.$id);
            const [item] = utils_1.addLajiFormIds(_getNewFormDataRow.call(this), tmpIdTree, false);
            return item;
        };
        const { onDropIndexClick } = this;
        this.onDropIndexClick = (index) => (event) => {
            const item = this.props.formData[index];
            const tmpIdTree = this.props.formContext.services.ids.getRelativeTmpIdTree(`${this.props.idSchema.$id}_${index}`);
            const oldIds = utils_1.getAllLajiFormIdsDeeply(item, tmpIdTree);
            Object.keys(oldIds).forEach((id) => {
                return this.props.formContext.services.submitHooks.remove(id);
            }, []);
            onDropIndexClick.call(this, index)(event);
        };
    }
    renderArrayFieldItem(props) {
        // rjsf@5 started rendering array items with titles as "name-idx" - we don't want that.
        if (typeof props.name === "string") {
            props.name = undefined;
        }
        if (this.props.idSchema) {
            const idSchema = this.getIdSchema(this.props, props.index);
            return super.renderArrayFieldItem(Object.assign(Object.assign({}, props), { itemIdSchema: idSchema }));
        }
        return super.renderArrayFieldItem(props);
    }
    getIdSchema(props, _index) {
        const { idSchema } = props;
        const index = utils_1.getFormDataIndex(_index, props.uiSchema);
        const root = idSchema.$id;
        const reduced = (idSchema, root) => Object.keys(idSchema).reduce((idSchema, prop) => {
            if (prop === "$id") {
                return Object.assign(Object.assign({}, idSchema), { "$id": idSchema.$id.replace(root, `${root}_${index}`) });
            }
            return Object.assign(Object.assign({}, idSchema), { [prop]: reduced(idSchema[prop], root) });
        }, idSchema);
        return reduced(idSchema, root);
    }
}
exports.ArrayFieldPatched = ArrayFieldPatched;
let _ArrayField = class _ArrayField extends React.Component {
    constructor() {
        super(...arguments);
        this.onChange = (formData) => {
            this.props.onChange(onArrayFieldChange(formData, this.props));
        };
        this.buttonDefinitions = {
            copy: {
                glyph: "duplicate",
                fn: () => (...params) => {
                    ArrayFieldTemplate_1.beforeAdd(this.props);
                    this.onChange([
                        ...this.props.formData,
                        exports.copyItemFunction(this, this.props.formData[this.props.formData.length - 1])(...params)
                    ]);
                },
                rules: {
                    minLength: 1,
                    canAdd: true
                },
                changesFormData: true
            },
            addPredefined: {
                fn: () => (onClickProps, { default: _default }) => {
                    ArrayFieldTemplate_1.beforeAdd(this.props);
                    this.onChange([
                        ...(this.props.formData || []),
                        utils_1.getDefaultFormState(this.props.schema.items, _default)
                    ]);
                },
                rules: {
                    canAdd: true
                },
                changesFormData: true
            }
        };
    }
    render() {
        const { props } = this;
        let { schema } = props;
        if (props.uiSchema.items && props.uiSchema.items["ui:field"]) {
            schema = Object.assign(Object.assign({}, schema), { uniqueItems: false });
        }
        // MultiArrayField needs to intercept default ArrayField internals, the instance is passed in formContext.
        const { ArrayField: _ArrayField } = props.formContext;
        const Component = _ArrayField || ArrayFieldPatched;
        // Reset formContext.ArrayField
        const formContext = _ArrayField ? Object.assign(Object.assign({}, props.formContext), { ArrayField: undefined }) : props.formContext;
        const registry = _ArrayField ? Object.assign(Object.assign({}, props.registry), { formContext }) : props.registry;
        return React.createElement(Component, Object.assign({}, props, { formContext: formContext, registry: registry, schema: schema, uiSchema: Object.assign(Object.assign({}, props.uiSchema), { "ui:options": Object.assign(Object.assign({ orderable: false }, props.uiSchema["ui:options"]), { buttonDefinitions: utils_1.getUiOptions(props.uiSchema).buttonDefinitions
                        ? merge(this.buttonDefinitions, utils_1.getUiOptions(props.uiSchema).buttonDefinitions)
                        : this.buttonDefinitions }) }), onChange: this.onChange }));
    }
};
_ArrayField.contextType = ReactContext_1.default;
_ArrayField = __decorate([
    BaseComponent_1.default
], _ArrayField);
exports.default = _ArrayField;
