"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
/**
 * Combines values of two fields into one value which can be used for displaying (editing that value doesn't change formData)
 * Combine types:
 * timeDifference: combines the values by calculating their time difference
 * stringJoin (default): combines the values by joining them. delimiter is added between if given
 */
const combinedPropType = PropTypes.shape({
    firstField: PropTypes.string.isRequired,
    secondField: PropTypes.string.isRequired,
    name: PropTypes.string,
    title: PropTypes.string,
    combineType: PropTypes.string,
    delimiter: PropTypes.string
});
let CombinedValueDisplayField = class CombinedValueDisplayField extends React.Component {
    constructor() {
        super(...arguments);
        this.toMinutes = (time) => {
            const parts = time.split(":");
            return Number(parts[0]) * 60 + Number(parts[1]);
        };
        this.onChange = (formData) => {
            const uiOptions = this.getUiOptions();
            const combined = Array.isArray(uiOptions.combined) ? uiOptions.combined : [uiOptions.combined];
            const combinedFields = combined.map(options => {
                return options.name || "";
            }, []);
            combinedFields.forEach(field => {
                formData = utils_1.immutableDelete(formData, field);
            });
            this.props.onChange(formData);
        };
    }
    static getName() { return "CombinedValueDisplayField"; }
    getStateFromProps(props) {
        const uiOptions = this.getUiOptions();
        let { schema, idSchema, formData } = props;
        const combined = Array.isArray(uiOptions.combined) ? uiOptions.combined : [uiOptions.combined];
        combined.forEach(options => {
            const { name, title, combineType, firstField, secondField } = options;
            schema = Object.assign(Object.assign({}, schema), { properties: Object.assign(Object.assign({}, schema.properties), { [name || ""]: { title: title || "", type: "string" } }) });
            idSchema = this.props.registry.schemaUtils.toIdSchema(schema, idSchema.$id);
            let value = undefined;
            const firstValue = firstField[0] === "/" ? utils_1.parseJSONPointer(formData, firstField, !!"safely") : formData[firstField];
            const secondValue = secondField[0] === "/" ? utils_1.parseJSONPointer(formData, secondField, !!"safely") : formData[secondField];
            if (combineType === "timeDifference") {
                if (firstValue && secondValue) {
                    const difference = this.toMinutes(secondValue) - this.toMinutes(formData[firstField]);
                    if (difference >= 0) {
                        const hours = Math.floor(difference / 60);
                        const minutes = difference % 60;
                        value = hours + " h " + minutes + " min";
                    }
                }
            }
            else {
                const delimiter = options.delimiter || "";
                value = [];
                if (firstValue)
                    value.push(firstValue);
                if (secondValue)
                    value.push(secondValue);
                value = value.join(delimiter);
            }
            formData = Object.assign(Object.assign({}, formData), { [name]: value });
        });
        return { schema, idSchema, formData, onChange: this.onChange };
    }
};
CombinedValueDisplayField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            combined: PropTypes.oneOfType([
                combinedPropType,
                PropTypes.arrayOf(combinedPropType)
            ])
        }),
        uiSchema: PropTypes.object
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object.isRequired
};
CombinedValueDisplayField = __decorate([
    VirtualSchemaField_1.default
], CombinedValueDisplayField);
exports.default = CombinedValueDisplayField;
