"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
const propsPropType = PropTypes.shape({
    from: PropTypes.string.isRequired,
    fromArrayKey: PropTypes.string,
    target: PropTypes.string,
    joinArray: PropTypes.bool
});
/**
 * Enables leaking any prop to any other prop container.
 *
 * uiSchema = {"ui:options": {
 *	props: [
 *		{
 *			from: If a regular string, a property to copy from formData. If a JSON pointer, the root will be this.props.
 *			target: Target where to copy the 'from'. If a regular string, it will be copied to /uischema/ui:options. If a JSON pointer, the root will be this.props.
 *		}
 */
let DataLeakerField = class DataLeakerField extends React.Component {
    static getName() { return "DataLeakerField"; }
    getStateFromProps(props) {
        const { props: _props = [] } = this.getUiOptions(props.uiSchema);
        return (Array.isArray(_props) ? _props : [_props]).reduce((props, strOrObjProp) => {
            const [fromPath, fromArrayKey, targetPath, joinArray] = ["from", "fromArrayKey", "target", "joinArray"]
                .map(p => strOrObjProp[p]);
            let from = fromPath[0] === "/"
                ? utils_1.parseJSONPointer(props, fromPath)
                : utils_1.parseJSONPointer(props.formData, fromPath);
            if (fromArrayKey) {
                from = (from || []).map(obj => obj[fromArrayKey]);
            }
            if (joinArray && Array.isArray(from)) {
                from = from.join(",");
            }
            const _targetPath = targetPath[0] === "/"
                ? targetPath
                : `/uiSchema/ui:options/${targetPath}`;
            props = utils_1.updateSafelyWithJSONPointer(props, from, _targetPath);
            return props;
        }, props);
    }
};
DataLeakerField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            props: PropTypes.oneOfType([PropTypes.arrayOf(propsPropType), propsPropType])
        }),
        uiSchema: PropTypes.object
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["array", "object"])
    }).isRequired,
    formData: PropTypes.oneOfType([PropTypes.array, PropTypes.object])
};
DataLeakerField = __decorate([
    VirtualSchemaField_1.default
], DataLeakerField);
exports.default = DataLeakerField;
