"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const immutability_helper_1 = require("immutability-helper");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
/**
 * Transforms an array field to o boolean field, where each value is true/false according to another array field.
 * Each value is true, if formData[booleanDefiner] == formData[booleanField].
 * uiSchema = { "ui:options": {
 *  booleanField: <array field name>,
 *  booleanDefiner: <array field name>,
 *  uiSchema: <uiSchema> (uiSchema used for each object).
 * }}
 */
let DependentBooleanField = class DependentBooleanField extends React.Component {
    constructor() {
        super(...arguments);
        this.onChange = (formData) => {
            const { booleanField, booleanDefiner } = this.getUiOptions();
            const propsFormData = this.props.formData;
            let newData = this.props.formData[booleanField];
            // if the change happened in booleanField data, reflect the changes to all booleanField items with same data.
            if (JSON.stringify(propsFormData[booleanDefiner]) === JSON.stringify(formData[booleanDefiner])) {
                let dictionarifiedOrigData = this.getDictionarifiedFormData(propsFormData, booleanField);
                formData[booleanDefiner].forEach((definerItem, i) => {
                    if (dictionarifiedOrigData[definerItem] && !formData[booleanField][i]) {
                        newData = immutability_helper_1.default(newData, { $splice: [[newData.indexOf(definerItem), 1]] });
                    }
                    else if (!dictionarifiedOrigData[definerItem] && formData[booleanField][i]) {
                        newData = newData ? [...newData, definerItem] : [definerItem];
                    }
                });
            }
            if (newData)
                newData = newData.filter(item => formData[booleanDefiner].includes(item));
            formData = Object.assign(Object.assign({}, formData), { [booleanField]: newData });
            this.props.onChange(formData);
        };
        this.getDictionarifiedFormData = (formData, field) => {
            let formDataDictionarified = {};
            (formData[field] || []).forEach((value) => {
                formDataDictionarified[value] = true;
            });
            return formDataDictionarified;
        };
        this.getDictionarifiedBooleanFieldData = (formData) => {
            return this.getDictionarifiedFormData(formData, this.getUiOptions().booleanField);
        };
    }
    static getName() { return "DependentBooleanField"; }
    getStateFromProps(props) {
        let { uiSchema, formData } = props;
        const { booleanField, booleanDefiner } = this.getUiOptions();
        let schema = immutability_helper_1.default(props.schema, { properties: { [booleanField]: { items: { $merge: { type: "boolean" } } } } });
        let booleanFieldDataDictionarified = this.getDictionarifiedBooleanFieldData(formData);
        let booleanFieldData = [];
        (formData[booleanDefiner] || []).forEach((definerItem) => {
            booleanFieldData.push(!!booleanFieldDataDictionarified[definerItem]);
        });
        formData = Object.assign(Object.assign({}, props.formData), { [booleanField]: booleanFieldData });
        return { schema, uiSchema, formData, onChange: this.onChange };
    }
};
DependentBooleanField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            booleanField: PropTypes.string.isRequired,
            booleanDefiner: PropTypes.string.isRequired,
        }).isRequired,
        uiSchema: PropTypes.object
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object.isRequired
};
DependentBooleanField = __decorate([
    VirtualSchemaField_1.default
], DependentBooleanField);
exports.default = DependentBooleanField;
