"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrayRulesPropType = void 0;
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const ConditionalUiSchemaField_1 = require("./ConditionalUiSchemaField");
const BaseComponent_1 = require("../BaseComponent");
const arrayRulePropType = PropTypes.shape({
    idx: PropTypes.number,
    isLast: PropTypes.number,
    complement: PropTypes.bool
});
exports.arrayRulesPropType = PropTypes.oneOfType([
    ConditionalUiSchemaField_1.rulePropType,
    arrayRulePropType,
    PropTypes.arrayOf(PropTypes.oneOfType([ConditionalUiSchemaField_1.rulePropType, arrayRulePropType]))
]);
let FilterArrayField = class FilterArrayField extends React.Component {
    constructor() {
        super(...arguments);
        this.getOnActiveChange = (idx, prop, callback) => {
            const offset = idx === undefined
                ? 0
                : (idx in this.filteredToOffsets)
                    ? this.filteredToOffsets[idx]
                    : this.filteredTotal;
            this.cachedOnActiveChange(idx === undefined ? undefined : idx + offset, prop, callback);
        };
        this.onChange = (formData) => {
            this.props.onChange(Object.keys(this.filteredIdxs).sort().reduce((_formData, idx) => {
                const updated = [..._formData];
                updated.splice(idx, 0, this.filteredIdxs[idx]);
                return updated;
            }, formData));
        };
    }
    getStateFromProps(props) {
        if (props.schema.items.type !== "object") {
            throw new Error("Can't use FilterArrayField if items aren't objects");
        }
        this.cache = this.cache || {};
        const { rules, cache } = utils_1.getUiOptions(props.uiSchema);
        this.filteredIdxs = {};
        this.filteredToOffsets = {};
        this.idxToOffsets = {};
        let filteredCount = 0;
        const store = (idx, item) => {
            this.filteredIdxs[idx] = item;
            filteredCount++;
        };
        const formData = (props.formData || []).filter((item, idx) => {
            let passes;
            if (cache) {
                const { passes: _passes, cache: _cache } = utils_1.checkArrayRules(rules, { formData: props.formData }, idx, this.cache);
                passes = _passes;
                this.cache = _cache;
                !passes && store(idx, item);
            }
            else {
                passes = utils_1.checkArrayRules(rules, { formData: props.formData }, idx);
                !passes && store(idx, item);
            }
            if (passes) {
                this.filteredToOffsets[idx - filteredCount] = filteredCount;
                this.filteredToOffsets[`_${idx}`] = filteredCount;
            }
            this.idxToOffsets[idx] = filteredCount;
            return passes;
        });
        this.filteredTotal = filteredCount;
        const errorSchema = Object.keys(props.errorSchema).reduce((errorSchema, idx) => {
            errorSchema[idx - this.idxToOffsets[idx]] = props.errorSchema[idx];
            return errorSchema;
        }, {});
        const innerUiSchema = utils_1.getInnerUiSchema(props.uiSchema);
        const uiSchema = Object.assign(Object.assign({}, innerUiSchema), { "ui:options": this.getActiveProps(Object.assign(Object.assign({}, utils_1.getUiOptions(innerUiSchema)), { idxOffsets: this.filteredToOffsets, totalOffset: this.filteredTotal })) });
        return Object.assign(Object.assign({}, props), { formData,
            uiSchema,
            errorSchema, onChange: this.onChange });
    }
    getActiveProps(options) {
        const { onActiveChange, activeIdx } = options;
        if (onActiveChange) {
            if (this.cachedOnActiveChange !== onActiveChange) {
                this.cachedOnActiveChange = onActiveChange;
            }
            options = Object.assign(Object.assign({}, options), { onActiveChange: this.getOnActiveChange });
        }
        if (activeIdx !== undefined) {
            options = Object.assign(Object.assign({}, options), { activeIdx: options.activeIdx - this.idxToOffsets[options.activeIdx] });
        }
        return options;
    }
    render() {
        const SchemaField = this.props.registry.fields.SchemaField;
        return React.createElement(SchemaField, Object.assign({}, this.props, this.state));
    }
};
FilterArrayField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            rules: exports.arrayRulesPropType,
            cache: PropTypes.boolean
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["array"])
    }).isRequired,
    formData: PropTypes.array
};
FilterArrayField = __decorate([
    BaseComponent_1.default
], FilterArrayField);
exports.default = FilterArrayField;
