import * as React from "react";
import * as PropTypes from "prop-types";
import { FieldProps } from "../LajiForm";
import ApiClient from "../../ApiClient";
interface ProcessedFile {
    dataURL: string;
    name: string;
    size: number;
    type: File["type"];
}
interface ImageArrayFieldState extends MediaArrayState {
    modalMediaSrc: string;
}
interface MediaMetadataSchema {
    capturerVerbatim?: string[];
    intellectualOwner?: string;
    intellectualRights?: string;
}
export default class ImageArrayField extends React.Component<FieldProps, ImageArrayFieldState> {
    ALLOWED_FILE_TYPES: string[];
    ACCEPT_FILE_TYPES: string[];
    MAX_FILE_SIZE: number;
    KEY: string;
    ENDPOINT: string;
    GLYPH: string;
    TRANSLATION_TAKE_NEW: string;
    TRANSLATION_SELECT_FILE: string;
    TRANSLATION_NO_MEDIA: string;
    CONTAINER_CLASS: string;
    METADATA_FORM_ID: string;
    renderMedia: (id: string) => JSX.Element;
    renderLoadingMedia: (id: string) => JSX.Element;
    onMediaClick: (i: number) => any;
    renderModalMedia: () => JSX.Element;
    formatValue(value: string[], options: any, props: FieldProps): string | null;
}
interface MediaArrayState {
    tmpMedias: number[];
    addModal?: any;
    dragging?: boolean;
    metadataModalOpen?: number | false;
    modalIdx?: number;
    modalMediaSrc?: string;
    modalMetadata?: any;
    metadataSaveSuccess?: string | false;
    metadataForm?: any;
    alert?: boolean;
    alertMsg?: string;
}
declare type Constructor<LFC> = new (...args: any[]) => LFC;
export declare function MediaArrayField<LFC extends Constructor<React.Component<FieldProps, MediaArrayState>>>(ComposedComponent: LFC): {
    new (...args: any[]): {
        ALLOWED_FILE_TYPES: string[];
        ACCEPT_FILE_TYPES: string[];
        MAX_FILE_SIZE: number;
        KEY: string;
        ENDPOINT: string;
        GLYPH: string;
        TRANSLATION_TAKE_NEW: string;
        TRANSLATION_SELECT_FILE: string;
        TRANSLATION_NO_MEDIA: string;
        CONTAINER_CLASS: string;
        METADATA_FORM_ID: string;
        deprecatedOptions: {
            imageAddModal: string;
            autoOpenImageAddModal: string;
        };
        apiClient: ApiClient;
        _context: any;
        mounted: boolean;
        fetching: any;
        addMediaContainerRef: React.RefObject<HTMLInputElement>;
        getOptions: (uiSchema: any) => any;
        componentDidMount(): void;
        componentWillUnmount(): void;
        onDragEnter: () => void;
        onDragLeave: () => void;
        onDrop: (files: File[]) => void;
        defaultOnClick: () => void;
        render(): JSX.Element;
        onKeyDown: (e: KeyboardEvent | React.KeyboardEvent<Element>) => void;
        renderMedias: () => any;
        renderLoadingMedias: () => (JSX.Element | null)[];
        openModalFor: (i: number) => () => void;
        onMediaRmClick: (i: number) => () => void;
        hideMetadataModal: () => void;
        onMetadataFormChange: (formData: any) => void;
        renderMetadataModal: () => JSX.Element | null;
        onHideMediaAddModal: () => void;
        renderMediaAddModal: () => JSX.Element | null;
        onAlertOk: () => void;
        parseExif: (files: File[]) => undefined | Promise<any>;
        sideEffects: (formData: any) => void;
        onFileFormChange: (files: File[]) => void;
        getContainerId: () => number | "root";
        saveMedias(files: File[]): Promise<any[] | undefined>;
        addNameToDataURL: (dataURL: string, name: string) => string;
        processFiles: (files: File[]) => Promise<ProcessedFile[]>;
        processFile: (file: File) => Promise<ProcessedFile>;
        onMediaMetadataUpdate: ({ formData }: {
            formData: any;
        }) => void;
        getMetadataPromise: () => Promise<MediaMetadataSchema>;
        getMaxFileSizeAsString: () => string;
        getAllowedMediaFormatsAsString: () => string;
        context: any;
        setState<K extends keyof MediaArrayState>(state: MediaArrayState | ((prevState: Readonly<MediaArrayState>, props: Readonly<FieldProps>) => MediaArrayState | Pick<MediaArrayState, K> | null) | Pick<MediaArrayState, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<FieldProps> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<MediaArrayState>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        shouldComponentUpdate?(nextProps: Readonly<FieldProps>, nextState: Readonly<MediaArrayState>, nextContext: any): boolean;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<FieldProps>, prevState: Readonly<MediaArrayState>): any;
        componentDidUpdate?(prevProps: Readonly<FieldProps>, prevState: Readonly<MediaArrayState>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<FieldProps>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<FieldProps>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<FieldProps>, nextState: Readonly<MediaArrayState>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<FieldProps>, nextState: Readonly<MediaArrayState>, nextContext: any): void;
    };
    contextType: React.Context<import("../../ReactContext").ContextProps>;
    propTypes: {
        uiSchema: PropTypes.Requireable<PropTypes.InferProps<{
            "ui:options": PropTypes.Requireable<PropTypes.InferProps<{
                titleClassName: PropTypes.Requireable<string>;
                addModal: PropTypes.Requireable<boolean | PropTypes.InferProps<{
                    labels: PropTypes.Requireable<PropTypes.InferProps<{
                        cancel: PropTypes.Requireable<string>;
                    }>>;
                }>>;
                autoOpenAddModal: PropTypes.Requireable<boolean>;
                autoOpenMetadataModal: PropTypes.Requireable<boolean>;
                sideEffects: PropTypes.Requireable<object>;
                exifParsers: PropTypes.Requireable<(object | null | undefined)[]>;
            }>>;
        }>>;
        schema: PropTypes.Validator<PropTypes.InferProps<{
            type: PropTypes.Requireable<string>;
            items: PropTypes.Validator<PropTypes.InferProps<{
                type: PropTypes.Validator<string>;
            }>>;
        }>>;
        formData: PropTypes.Requireable<any[]>;
    };
    displayName: string;
} & LFC;
export {};
