"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const immutability_helper_1 = require("immutability-helper");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
/**
 * Injects given fields value as default value to target field.
 * uiSchema = { "ui:options": {
 *  "injections": [
 *    {
 *      "fields": [field1, field2...], (fields to inject from source field)
 *      "target": fieldName (target field where default value is injected)
 *      "source": fieldName (source field where default value is injected from. Must be object field)
 *    }
 *    ...
 *  ]
 * }}
 */
let InjectDefaultValueField = class InjectDefaultValueField extends React.Component {
    constructor() {
        super(...arguments);
        this.onChange = (formData) => {
            let { schema } = this.props;
            const options = this.getUiOptions();
            const { fields, target } = options;
            let source = options.source ? formData[options.source] : formData;
            fields.forEach(field => {
                if (schema.properties[target].type === "array") {
                    if (formData && formData[target])
                        formData[target].forEach((item, i) => {
                            if (item[field] === this.props.formData[options.source][field]) {
                                formData = immutability_helper_1.default(formData, { [target]: { $splice: [[i, 1, immutability_helper_1.default(item, { $merge: { [field]: source[field] } })]] } });
                            }
                        });
                }
            });
            this.props.onChange(formData);
        };
    }
    static getName() { return "InjectDefaultValueField"; }
    getStateFromProps() {
        return { onChange: this.onChange };
    }
};
InjectDefaultValueField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            fields: PropTypes.arrayOf(PropTypes.string).isRequired,
            target: PropTypes.string.isRequired,
            source: PropTypes.string.isRequired
        }).isRequired
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object", "array"])
    }).isRequired,
    formData: PropTypes.oneOfType([PropTypes.object, PropTypes.array]).isRequired
};
InjectDefaultValueField = __decorate([
    VirtualSchemaField_1.default
], InjectDefaultValueField);
exports.default = InjectDefaultValueField;
