"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const ConditionalUiSchemaField_1 = require("./ConditionalUiSchemaField");
const FilterArrayField_1 = require("./FilterArrayField");
const utils_1 = require("../../utils");
const BaseComponent_1 = require("../BaseComponent");
const Context_1 = require("../../Context");
const ArrayFieldTemplate_1 = require("../templates/ArrayFieldTemplate");
let MultiArrayField = class MultiArrayField extends React.Component {
    constructor() {
        super(...arguments);
        this.itemIds = {};
        this.onChange = (idx) => (formData) => {
            let offset = 0;
            for (let i = 0; i < idx; i++) {
                offset += Object.keys(this.groupedItems[i]).length;
            }
            let { persistenceKey } = utils_1.getUiOptions(this.props.uiSchema);
            const lengthChange = formData.length - this.groupedItems[idx].length;
            const context = Context_1.default(`${persistenceKey}_MULTI`);
            // Fix idxs map for items after this group.
            if (lengthChange && context.nonGroupedMap) {
                const changes = {};
                const changeFrom = {};
                for (let i = offset + Object.keys(this.groupedItems[idx]).length; i < (this.props.formData || []).length; i++) {
                    if (context.nonGroupedMap[i]) {
                        changes[i + lengthChange] = context.nonGroupedMap[i];
                        changeFrom[i] = true;
                    }
                }
                context.nonGroupedMap = Object.assign(Object.assign({}, context.nonGroupedMap), changes);
                Object.keys(changeFrom).filter(idx => !(idx in changes)).forEach(idx => {
                    delete context.nonGroupedMap[idx];
                });
            }
            if (typeof persistenceKey === "string" && formData.length !== this.groupedItems[idx].length) {
                if (!context.nonGroupedMap) {
                    context.nonGroupedMap = {};
                }
                context.nonGroupedMap[offset + formData.length - 1] = idx;
            }
            formData.forEach(item => {
                this.groupItemIds[idx][utils_1.getUUID(item)] = true;
            });
            const newFormData = this.groupedItems.reduce((flat, groupItems, _idx) => {
                if (_idx === idx)
                    return [...flat, ...formData];
                return [...flat, ...groupItems];
            }, []);
            this.props.onChange(newFormData);
        };
    }
    render() {
        const { props } = this;
        let { groups, cache = true, arrayMerge, persistByParent, persistenceKey, renderNonGrouped = false, nonGroupedOperations } = utils_1.getUiOptions(this.props.uiSchema);
        if (groups.length && !this.cache) {
            this.cache = Array(groups.length).fill(undefined).map(_ => ({})); // eslint-disable-line @typescript-eslint/no-unused-vars
        }
        if (groups.length) {
            this.arrayKeyFunctions = Array(groups.length + 1).fill(undefined).map((_, idx) => getArrayKeyFunctions(this, idx));
        }
        if (groups.length && !this.groupItemIds) {
            const getGroupItemIds = () => Array(groups.length + 1).fill(undefined).map(_ => ({})); // eslint-disable-line @typescript-eslint/no-unused-vars
            if (persistByParent) {
                const parentId = this.props.formContext._parentLajiFormId;
                const context = Context_1.default(`${parentId}_MULTI`);
                this.groupItemIds = context.groupItemIds || getGroupItemIds();
                context.groupItemIds = this.groupItemIds;
            }
            else if (typeof persistenceKey === "string") {
                const context = Context_1.default(`${persistenceKey}_MULTI`);
                this.groupItemIds = context.groupItemIds || getGroupItemIds();
                context.groupItemIds = this.groupItemIds;
            }
            else {
                this.groupItemIds = getGroupItemIds();
            }
        }
        const itemGroups = Array(groups.length + 1).fill(undefined).map(_ => []); // eslint-disable-line @typescript-eslint/no-unused-vars
        const addToGroup = (idx, item) => {
            const id = utils_1.getUUID(item);
            itemGroups[idx].push(item);
            this.groupItemIds[idx][id] = true;
            this.itemIds[id] = true;
            return true;
        };
        (props.formData || []).forEach(item => {
            this.groupItemIds.forEach((_, groupIdx) => {
                if (this.groupItemIds[groupIdx][utils_1.getUUID(item)]) {
                    addToGroup(groupIdx, item);
                }
            });
        });
        const nonGrouped = [];
        const groupedItems = (props.formData || []).reduce((itemGroups, item, idx) => {
            const addedToGroup = groups.some((group, groupIdx) => {
                if (this.itemIds[utils_1.getUUID(item)]) {
                    return true;
                }
                const { rules = [] } = group;
                let passes;
                if (cache) {
                    const check = utils_1.checkArrayRules(rules, { formData: props.formData || [] }, idx, this.cache[groupIdx]);
                    const { cache } = check;
                    passes = check.passes;
                    this.cache[groupIdx] = cache;
                }
                else {
                    passes = utils_1.checkArrayRules(rules, { formData: props.formData || [] }, idx);
                }
                if (passes) {
                    return addToGroup(groupIdx, item);
                }
            });
            if (!addedToGroup) {
                nonGrouped.push([item, idx]);
            }
            return itemGroups;
        }, itemGroups);
        nonGrouped.forEach(([item, idx]) => {
            const context = Context_1.default(`${persistenceKey}_MULTI`);
            const groupIdx = this.groupItemIds.length - 1;
            if (typeof persistenceKey === "string" && (context.nonGroupedMap || {})[idx] !== undefined && this.groupItemIds[groupIdx]) {
                addToGroup(context.nonGroupedMap[idx], item);
            }
            else {
                addToGroup(groupIdx, item);
            }
        });
        this.groupedItems = groupedItems;
        const withoutNonGrouped = [...groupedItems];
        withoutNonGrouped.pop();
        let offset = 0;
        const offsets = (renderNonGrouped ? groupedItems : withoutNonGrouped).map((group, idx) => {
            let idxOffsets;
            if (!idx) {
                idxOffsets = {};
            }
            else {
                idxOffsets = group.reduce((g, i, _idx) => {
                    g[_idx] = offset;
                    return g;
                }, {});
            }
            offset += group.length;
            return idxOffsets;
        }, []);
        return (renderNonGrouped ? groupedItems : withoutNonGrouped).map((items, idx) => {
            const operations = renderNonGrouped && idx === groupedItems.length - 1
                ? nonGroupedOperations
                : groups[idx].operations;
            const innerUiSchema = utils_1.getInnerUiSchema(props.uiSchema);
            let uiSchema = operations
                ? ConditionalUiSchemaField_1.computeUiSchema(innerUiSchema, operations, arrayMerge)
                : innerUiSchema;
            uiSchema = utils_1.updateSafelyWithJSONPointer(uiSchema, this.arrayKeyFunctions[idx], "/ui:options/arrayKeyFunctions");
            uiSchema = utils_1.updateSafelyWithJSONPointer(uiSchema, offsets[idx], "/ui:options/idxOffsets");
            let offset = 0;
            for (let i = 0; i < idx; i++) {
                offset += this.groupedItems[i].length;
            }
            uiSchema = Object.assign(Object.assign({}, uiSchema), { "ui:options": Object.assign(Object.assign({}, utils_1.getUiOptions(uiSchema)), { startIdx: offset }) });
            const errorSchema = {};
            for (const key in props.errorSchema) {
                if (key >= offset && key < offset + this.groupedItems[idx].length) {
                    errorSchema[key - offset] = Object.assign({}, props.errorSchema[key]);
                }
            }
            const { SchemaField } = this.props.registry.fields;
            return (React.createElement(SchemaField, Object.assign({ key: idx }, props, { uiSchema: uiSchema, errorSchema: errorSchema, formData: items, onChange: this.onChange(idx) })));
        });
    }
};
MultiArrayField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            rules: FilterArrayField_1.arrayRulesPropType,
            itemsOperations: PropTypes.oneOfType([
                ConditionalUiSchemaField_1.operationPropType,
                PropTypes.arrayOf(ConditionalUiSchemaField_1.operationPropType)
            ]),
            cache: PropTypes.boolean
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["array"])
    }).isRequired,
    formData: PropTypes.array
};
MultiArrayField = __decorate([
    BaseComponent_1.default
], MultiArrayField);
exports.default = MultiArrayField;
const getArrayKeyFunctions = (that) => {
    const { arrayKeyFunctions: optionsArrayKeyFunctions } = utils_1.getUiOptions(that.props.uiSchema);
    const _arrayKeyFunctions = optionsArrayKeyFunctions || ArrayFieldTemplate_1.arrayKeyFunctions;
    return Object.assign(Object.assign({}, _arrayKeyFunctions), { insert: (e, _props) => {
            const props = _props.getProps();
            const inputElem = utils_1.findNearestParentTabbableElem(document.activeElement);
            if (!inputElem) {
                return false;
            }
            if (!inputElem.id.startsWith(props.idSchema.$id)) {
                return false;
            }
            const itemIdx = inputElem.id.replace(props.idSchema.$id, "").replace(/^_?([0-9]+).*$/, "$1");
            const { startIdx } = utils_1.getUiOptions(props.uiSchema);
            if (itemIdx < startIdx || itemIdx >= startIdx + (props.formData || []).length) {
                return false;
            }
            return _arrayKeyFunctions.insert(e, _props);
        }, navigateArray: (e, options) => {
            const _options = Object.assign(Object.assign({}, options), { getProps: () => (Object.assign(Object.assign({}, options.getProps()), { formData: that.props.formData })) });
            return _arrayKeyFunctions.navigateArray(e, _options);
        } });
};
