"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const BaseComponent_1 = require("../BaseComponent");
const utils_1 = require("../../utils");
const ReactContext_1 = require("../../ReactContext");
const components_1 = require("../components");
const immutability_helper_1 = require("immutability-helper");
const memoize = require("memoizee");
const react_dom_1 = require("react-dom");
let MultiTagArrayField = class MultiTagArrayField extends React.Component {
    constructor() {
        super(...arguments);
        this.state = { activeButtonIdx: undefined };
        this.affixContainerElem = null;
        this.setAffixContainer = (ref) => {
            this.affixContainerElem = react_dom_1.findDOMNode(ref);
        };
        this.getAffixContainer = () => {
            return this.affixContainerElem;
        };
        this.onChange = memoize((key) => (formData) => {
            const newFormData = Object.assign(Object.assign({}, this.props.formData), { [key]: formData });
            this.props.onChange(newFormData);
        });
        this.onButtonClick = memoize((idx) => () => {
            const { activeButtonIdx } = this.state;
            if (activeButtonIdx === idx) {
                this.setState({ activeButtonIdx: undefined });
                return;
            }
            const { buttons = [] } = utils_1.getUiOptions(this.props.uiSchema);
            const options = buttons[idx];
            const { operation } = options;
            if (operation === "moveAll") {
                const { toField, fromField } = options;
                const toFieldNewValue = (this.props.formData[toField] || []).concat(this.props.formData[fromField] || []);
                const newFormData = Object.assign(Object.assign({}, this.props.formData), { [fromField]: [], [toField]: toFieldNewValue });
                this.props.onChange(newFormData);
            }
            else if (operation === "move" || operation === "delete") {
                this.setState({ activeButtonIdx: idx });
            }
        });
        this.onTagClick = memoize((fromField) => (idx) => {
            const { activeButtonIdx } = this.state;
            if (activeButtonIdx === undefined) {
                return;
            }
            const { buttons = [] } = utils_1.getUiOptions(this.props.uiSchema);
            const options = buttons[activeButtonIdx];
            const { operation } = options;
            if (operation === "move" || operation === "delete") {
                let formData = this.props.formData;
                if (operation === "move") {
                    const { toField } = options;
                    if (fromField === toField) {
                        return;
                    }
                    const value = formData[fromField][idx];
                    formData = immutability_helper_1.default(formData, {
                        [fromField]: { $splice: [[idx, 1]] },
                        [toField]: { $apply: (values) => [...(values || []), value] }
                    });
                }
                else {
                    formData = immutability_helper_1.default(formData, { [fromField]: { $splice: [[idx, 1]] } });
                }
                this.props.onChange(formData);
            }
        });
    }
    render() {
        const SchemaField = this.props.registry.fields.SchemaField;
        const { Row, Col } = this.context.theme;
        const { schema, uiSchema, idSchema, errorSchema, formData = {} } = this.props;
        const uiOptions = utils_1.getUiOptions(uiSchema);
        const { buttons = [] } = uiOptions;
        const { activeButtonIdx } = this.state;
        const propertyKeys = Object.keys(schema.properties);
        return (React.createElement(Row, { className: utils_1.classNames("laji-form-multi-tag-array-field", activeButtonIdx && "laji-form-multi-tag-array-field-active"), ref: this.setAffixContainer },
            React.createElement(Col, { xs: 3, sm: 3, md: 2, lg: 2, className: "laji-form-multi-tag-array-field-buttons" },
                React.createElement(components_1.Affix, { getContainer: this.getAffixContainer, topOffset: this.props.formContext.topOffset + 15, bottomOffset: this.props.formContext.bottomOffset },
                    React.createElement("div", { className: "btn-group-vertical" }, buttons.map((btnProps, idx) => (React.createElement(components_1.Button, { key: idx, onClick: this.onButtonClick(idx), variant: btnProps.operation === "delete" ? "outline-danger" : "default", className: utils_1.classNames(btnProps.className, activeButtonIdx === idx ? "active" : "") }, btnProps.label)))))),
            React.createElement(Col, { xs: 9, sm: 9, md: 10, lg: 10, className: "laji-form-multi-tag-array-field-content" }, propertyKeys.map(key => {
                var _a;
                return (React.createElement(SchemaField, Object.assign({}, this.props, { key: key, schema: Object.assign({ title: "" }, schema.properties[key]), uiSchema: {
                        "ui:field": "TagArrayField",
                        "ui:options": Object.assign(Object.assign({}, uiOptions), { buttons: undefined, onTagClick: this.onTagClick(key) })
                    }, errorSchema: errorSchema[key] || {}, idSchema: idSchema[key], formData: formData[key] || [], required: ((_a = uiSchema[key]) === null || _a === void 0 ? void 0 : _a["ui:required"]) || false, onChange: this.onChange(key) })));
            }))));
    }
};
MultiTagArrayField.contextType = ReactContext_1.default;
MultiTagArrayField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            buttons: PropTypes.arrayOf(PropTypes.oneOfType([
                PropTypes.shape({
                    operation: PropTypes.oneOf(["moveAll"]),
                    fromField: PropTypes.string.isRequired,
                    toField: PropTypes.string.isRequired,
                    label: PropTypes.string,
                    className: PropTypes.string
                }),
                PropTypes.shape({
                    operation: PropTypes.oneOf(["move"]),
                    toField: PropTypes.string.isRequired,
                    label: PropTypes.string,
                    className: PropTypes.string
                }),
                PropTypes.shape({
                    operation: PropTypes.oneOf(["delete"]),
                    label: PropTypes.string,
                    className: PropTypes.string
                })
            ]))
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object
};
MultiTagArrayField = __decorate([
    BaseComponent_1.default
], MultiTagArrayField);
exports.default = MultiTagArrayField;
