"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_dom_1 = require("react-dom");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const components_1 = require("../components");
const Context_1 = require("../../Context");
const ReactContext_1 = require("../../ReactContext");
const BaseComponent_1 = require("../BaseComponent");
const Spinner = require("react-spinner");
const MapArrayField_1 = require("./MapArrayField");
const SAVE = "SAVE", FETCH = "FETCH";
/**
 * Compatible only with gathering field.
 */
let NamedPlaceSaverField = class NamedPlaceSaverField extends React.Component {
    constructor() {
        super(...arguments);
        this.onButtonClick = () => () => {
            this.setState({ show: !this.state.show });
        };
        this.onSave = (place) => {
            this.setState({ show: false }, () => {
                this.props.onChange(Object.assign(Object.assign({}, (this.props.formData || {})), { namedPlaceID: place.id }));
            });
        };
        this.onHide = () => this.setState({ show: false });
    }
    getStateFromProps(props) {
        const innerUiSchema = utils_1.getInnerUiSchema(props.uiSchema);
        const uiSchema = Object.assign(Object.assign({}, innerUiSchema), { "ui:options": Object.assign(Object.assign({}, utils_1.getUiOptions(innerUiSchema)), { buttons: [
                    ...(utils_1.getUiOptions(innerUiSchema).buttons || []),
                    this.getButton(props)
                ] }) });
        return { uiSchema };
    }
    getButton(props) {
        return {
            label: props.formContext.translations.SaveNamedPlace,
            fn: this.onButtonClick,
            glyph: "floppy-disk",
            position: "top",
            disabled: !MapArrayField_1.parseGeometries((props.formData || {}).geometry).length
        };
    }
    componentDidUpdate(prevProps, prevState) {
        if (prevState.show !== this.state.show)
            this.setState(this.getStateFromProps(this.props));
    }
    render() {
        const { registry: { fields: { SchemaField } }, formContext } = this.props;
        const { uiSchema } = this.state;
        const { Modal } = this.context.theme;
        return (React.createElement("div", null,
            React.createElement(SchemaField, Object.assign({}, this.props, { uiSchema: uiSchema })),
            this.state && this.state.show
                ? (React.createElement(Modal, { dialogClassName: "laji-form", show: true, onHide: this.onHide },
                    React.createElement(Modal.Header, null,
                        React.createElement(Modal.Title, null, formContext.translations.NamedPlaces)),
                    React.createElement(Modal.Body, null,
                        React.createElement(PlaceSaverDialog, { formContext: formContext, onSave: this.onSave, gathering: this.props.formData || {} })))) : null));
    }
};
NamedPlaceSaverField.contextType = ReactContext_1.default;
NamedPlaceSaverField.propTypes = {
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object
};
NamedPlaceSaverField = __decorate([
    BaseComponent_1.default
], NamedPlaceSaverField);
exports.default = NamedPlaceSaverField;
class PlaceSaverDialog extends React.Component {
    constructor(props) {
        super(props);
        this.onInputChange = ({ target: { value } }) => {
            this.setState({ value });
        };
        this.getGathering = () => {
            return ["geometry", "country", "administrativeProvince", "biologicalProvince", "municipality", "locality", "localityDescription", "habitat", "habitatDescription", "coordinateRadius"].reduce((gathering, key) => {
                gathering[key] = this.props.gathering[key];
                return gathering;
            }, {});
        };
        this.onSaveNew = (e) => {
            e.preventDefault();
            const _a = this.getGathering(), { namedPlaceID } = _a, gathering = __rest(_a, ["namedPlaceID"]); //eslint-disable-line @typescript-eslint/no-unused-vars
            this.onSave({
                name: this.state.value,
                prepopulatedDocument: { gatherings: [gathering] }
            });
        };
        this.onOverwriteCurrent = () => {
            const { namedPlaceID } = this.props.gathering;
            const gathering = this.getGathering(); //eslint-disable-line no-unused-vars
            this.onSave(Object.assign(Object.assign({}, this.state.placeIdsToPlaces[namedPlaceID]), { name: this.state.value, prepopulatedDocument: { gatherings: [gathering] } }));
        };
        this.onOverwriteSelected = (place) => () => {
            const gathering = this.getGathering(); //eslint-disable-line no-unused-vars
            this.onSave(Object.assign(Object.assign({}, place), { name: this.state.value, prepopulatedDocument: { gatherings: [gathering] } }));
        };
        this.onOverwriteExisting = () => {
            this.onOverwriteSelected(this.state.placeNamesToPlaces[this.state.value])();
        };
        // Hack that fixed autofocus cursor position
        this.onFocus = (e) => {
            var val = e.target.value;
            e.target.value = "";
            e.target.value = val;
        };
        this.onSubmit = (e) => {
            e.preventDefault();
        };
        this.setInputRef = (elem) => {
            this.inputRef = elem;
        };
        const { namedPlaceID, locality = "" } = props.gathering;
        this.state = { value: locality };
        if (namedPlaceID) {
            this.state = { value: "", loading: true };
        }
        this.apiClient = props.formContext.apiClient;
    }
    componentWillUnmount() {
        this.mounted = false;
    }
    componentDidMount() {
        this.mounted = true;
        this.apiClient.fetchCached("/named-places", { includePublic: false, pageSize: 1000 }).then(response => {
            if (!this.mounted)
                return;
            const state = {
                places: response.results,
                placeIdsToPlaces: response.results.reduce((idsToPlaces, place) => {
                    idsToPlaces[place.id] = place;
                    return idsToPlaces;
                }, {}),
                placeNamesToPlaces: response.results.reduce((namesToPlaces, place) => {
                    if (!namesToPlaces[place.name])
                        namesToPlaces[place.name] = [];
                    namesToPlaces[place.name].push(place);
                    return namesToPlaces;
                }, {})
            };
            const { namedPlaceID } = this.props.gathering;
            if (namedPlaceID) {
                state.value = state.placeIdsToPlaces[namedPlaceID].name;
                state.loading = false;
            }
            this.setState(state, () => {
                if (namedPlaceID) {
                    react_dom_1.findDOMNode(this.inputRef).focus();
                }
            });
        }).catch(() => {
            this.setState({ failed: FETCH });
        });
    }
    onSave(place) {
        place = Object.assign(Object.assign({}, place), { geometry: place.prepopulatedDocument.gatherings[0].geometry });
        this.props.formContext.services.blocker.push();
        this.apiClient.fetchRaw(`/named-places${place.id ? `/${place.id}` : ""}`, undefined, {
            method: place.id ? "PUT" : "POST",
            headers: {
                "accept": "application/json",
                "content-type": "application/json"
            },
            body: JSON.stringify(place)
        }).then(response => {
            this.props.formContext.services.blocker.pop();
            this.apiClient.invalidateCachePath("/named-places");
            return response.json();
        }).then(this.props.onSave)
            .catch(() => {
            this.props.formContext.services.blocker.pop();
            this.setState({ failed: SAVE });
        });
    }
    render() {
        const { value, placeNamesToPlaces = {}, loading } = this.state;
        const { translations } = this.props.formContext;
        const existingPlaces = (placeNamesToPlaces[value] || []).filter(place => {
            return (place.owners || []).includes(this.props.formContext.uiSchemaContext.creator);
        });
        const getButton = (onClick, text) => {
            return React.createElement(components_1.Button, { small: true, onClick: onClick, disabled: loading || utils_1.isEmptyString(value) }, text);
        };
        const { Panel, FormGroup, FormControl, ListGroup, ListGroupItem, Alert, Form, ControlLabel } = this.context.theme;
        return this.state.failed ? (React.createElement(Alert, { variant: "danger" }, `${translations[`NamedPlaces${this.state.failed === FETCH ? "Fetch" : "Save"}Fail`]} ${translations.TryAgainLater}`)) : (React.createElement("div", { className: "place-saver-dialog" },
            React.createElement(Form, { inline: true, onSubmit: this.onSubmit },
                React.createElement(FormGroup, null,
                    React.createElement(ControlLabel, null,
                        translations.Name,
                        ": "),
                    " ",
                    React.createElement(FormControl, { type: "text", value: value, onChange: this.onInputChange, autoFocus: true, onFocus: this.onFocus, disabled: loading, ref: this.setInputRef }),
                    " ",
                    !existingPlaces.length ?
                        getButton(this.onSaveNew, `${translations.Save} ${translations.new}`) : null,
                    " ",
                    this.props.gathering.namedPlaceID ?
                        getButton(this.onOverwriteCurrent, translations.SaveCurrentOverwrite) : null,
                    existingPlaces.length === 1 && !this.props.gathering.namedPlaceID ?
                        getButton(this.onOverwriteExisting, translations.SaveExistingOverwrite) : null),
                loading ? React.createElement("div", { className: "pull-right" },
                    React.createElement(Spinner, null)) : null),
            !this.props.gathering.namedPlaceID && existingPlaces.length ?
                React.createElement(Alert, { variant: "warning" }, `${translations.Warning}: ${translations.UsedNamedPlaceName}.${existingPlaces.length > 1 ? ` ${translations.UsedNamedPlaceNameMultiple}.` : ""}`)
                : null,
            existingPlaces.length > 1 ? (React.createElement(FormGroup, null,
                React.createElement(Panel, null,
                    React.createElement(Panel.Heading, null, translations.ClickPlaceToOverwrite),
                    React.createElement(ListGroup, { fill: "fill" }, existingPlaces.map(place => React.createElement(ListGroupItem, { header: place.name, key: place.id, onClick: loading ? undefined : this.onOverwriteSelected(place), disabled: loading }, place.notes)))))) : null));
    }
}
PlaceSaverDialog.contextType = ReactContext_1.default;
Context_1.default("SCHEMA_FIELD_WRAPPERS").NamedPlaceSaverField = true;
