"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagInputComponent = void 0;
const React = require("react");
const react_dom_1 = require("react-dom");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const BaseComponent_1 = require("../BaseComponent");
const memoize = require("memoizee");
let TagArrayField = class TagArrayField extends React.Component {
    render() {
        const { FieldTemplate } = this.props.registry.templates;
        const { uiSchema } = this.props;
        return (React.createElement(FieldTemplate, Object.assign({}, this.props, { forceDisplayLabel: true, rawHelp: uiSchema["ui:help"], description: uiSchema["ui:description"], rawErrors: [] }),
            React.createElement(TagInputComponent, Object.assign({}, this.props, { id: this.props.idSchema.$id, tags: this.props.formData }))));
    }
};
TagArrayField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            separatorKeys: PropTypes.arrayOf(PropTypes.string),
            showDeleteButton: PropTypes.bool
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["array"])
    }).isRequired,
    formData: PropTypes.array
};
TagArrayField = __decorate([
    BaseComponent_1.default
], TagArrayField);
exports.default = TagArrayField;
class TagInputComponent extends React.Component {
    constructor(props) {
        super(props);
        this.getStateFromProps = ({ value }) => {
            return { value };
        };
        this.getSeparatorKeys = (uiSchema) => {
            const { separatorKeys = ["Enter", ",", ";"] } = utils_1.getUiOptions(uiSchema);
            return separatorKeys;
        };
        this.onKeyDown = (e) => {
            const value = this.getTrimmedValue();
            const { tags = [] } = this.props;
            const separatorKeys = this.getSeparatorKeys(this.props.uiSchema);
            if (separatorKeys.includes(e.key) && !utils_1.isEmptyString(value)) {
                this.props.onChange([...tags, value], "enter");
                e.stopPropagation();
                e.preventDefault();
            }
            else if (e.key === "Backspace" && utils_1.isEmptyString(value) && tags.length) {
                this.onRemove(tags.length - 1)();
            }
            utils_1.triggerParentComponent("onKeyDown", e, this.props);
        };
        this.onRemove = memoize((idx) => () => {
            const tags = [...(this.props.tags || [])];
            tags.splice(idx, 1);
            this.props.onChange(tags, "remove");
        });
        this.onFocus = (e) => {
            this.setState({ focused: true }, () => {
                utils_1.triggerParentComponent("onFocus", e, this.props);
            });
        };
        this.onBlur = (e) => {
            this.setState({ focused: false });
            utils_1.triggerParentComponent("onBlur", e, this.props);
            const value = this.getTrimmedValue();
            if (!utils_1.isEmptyString(value)) {
                this.props.onChange([...(this.props.tags || []), value], "blur");
            }
        };
        this.setInputRef = (ref) => {
            this.inputRef = ref;
        };
        this.onClick = () => {
            react_dom_1.findDOMNode(this.inputRef).focus();
        };
        this.onInputChange = (e) => {
            const { onInputChange } = this.props;
            onInputChange && e.persist();
            const { target: { value } } = e;
            const separatorKeys = this.getSeparatorKeys(this.props.uiSchema);
            const splitted = separatorKeys.reduce((splitted, separator) => splitted.reduce((_splitted, i) => ([..._splitted, ...i.split(separator)]), []), [value]).map(s => s.trim()).filter(s => !utils_1.isEmptyString(s));
            this.setState({ value }, () => {
                onInputChange && this.props.onInputChange(e);
                if (splitted.length > 1) {
                    this.props.onChange([...(this.props.tags || []), ...splitted]);
                }
            });
        };
        this.onTagClick = memoize((idx) => () => {
            const { onTagClick } = utils_1.getUiOptions(this.props.uiSchema);
            onTagClick === null || onTagClick === void 0 ? void 0 : onTagClick(idx);
        });
        this.state = this.getStateFromProps(props);
    }
    UNSAFE_componentWillReceiveProps(props) {
        this.setState(this.getStateFromProps(props));
    }
    getTrimmedValue() {
        const { value } = this.state;
        return value === null || value === void 0 ? void 0 : value.trim();
    }
    render() {
        let { tags = [], InputComponent, readonly, disabled, uiSchema } = this.props;
        tags = tags.filter(s => !utils_1.isEmptyString(s));
        const { value = "" } = this.state;
        const inputProps = Object.assign(Object.assign({ type: "text", className: "rw-input-reset", ref: this.setInputRef, value: value, onChange: this.onInputChange, id: this.props.id }, this.props.inputProps), { onFocus: this.onFocus, onBlur: this.onBlur, onKeyDown: this.onKeyDown });
        const { showDeleteButton = true } = utils_1.getUiOptions(uiSchema);
        return (React.createElement("div", { className: `rw-multiselect rw-widget${this.state.focused ? " rw-state-focus" : ""}${readonly || disabled ? " rw-state-disabled" : ""}`, onClick: this.onClick },
            React.createElement("div", { className: "rw-widget-input rw-widget-picked rw-widget-container" },
                React.createElement("ul", { className: "rw-multiselect-taglist" }, tags.map((item, idx) => React.createElement("li", { key: idx, className: "rw-multiselect-tag", onClick: this.onTagClick(idx) },
                    item,
                    showDeleteButton ? React.createElement("span", { className: "rw-tag-btn", onClick: this.onRemove(idx), tabIndex: 0, onKeyDown: this.props.formContext.utils.keyboardClick(this.onRemove(idx)) }, "\u00D7") : ""))),
                InputComponent ? React.createElement(InputComponent, Object.assign({}, inputProps)) : React.createElement("input", Object.assign({}, inputProps, { disabled: disabled || readonly })))));
    }
}
exports.TagInputComponent = TagInputComponent;
