"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const VirtualSchemaField_1 = require("../VirtualSchemaField");
const BaseComponent_1 = require("../BaseComponent");
const components_1 = require("../components");
const merge = require("deepmerge");
const ReactContext_1 = require("../../ReactContext");
let UnitCountShorthandField = class UnitCountShorthandField extends React.Component {
    constructor(props) {
        super(props);
        this.parseCode = this.parseCode.bind(this);
    }
    static getName() { return "UnitCountShorthandField"; }
    getStateFromProps(props) {
        const { uiSchema, formData } = props;
        const { shorthandField, taxonIDField } = utils_1.getUiOptions(props.uiSchema);
        const shortHandUiSchema = merge(utils_1.parseJSONPointer(props.uiSchema, shorthandField) || {}, {
            "ui:widget": CodeReader,
            "ui:options": {
                parseCode: this.parseCode,
                taxonID: utils_1.parseJSONPointer(formData, taxonIDField)
            }
        });
        const _uiSchema = utils_1.updateSafelyWithJSONPointer(uiSchema, shortHandUiSchema, shorthandField);
        return Object.assign(Object.assign({}, props), { uiSchema: _uiSchema });
    }
    parseCode(value, taxonId) {
        const { apiClient } = this.props.formContext;
        let formData = this.props.formData;
        const { shorthandField, pairCountField } = utils_1.getUiOptions(this.props.uiSchema);
        let timestamp = Date.now();
        this.promiseTimestamp = timestamp;
        return new Promise((resolve, reject) => {
            if (!value || !taxonId) {
                formData = utils_1.updateSafelyWithJSONPointer(formData, undefined, pairCountField);
                this.props.onChange(formData);
                resolve({ success: undefined });
                return;
            }
            apiClient.fetchCached("/autocomplete/pairCount", { q: value, taxonID: taxonId }).then(suggestion => {
                if (timestamp !== this.promiseTimestamp) {
                    reject();
                    return;
                }
                formData = utils_1.updateSafelyWithJSONPointer(formData, suggestion.key, shorthandField);
                formData = utils_1.updateSafelyWithJSONPointer(formData, suggestion.value, pairCountField);
                this.props.onChange(formData);
                resolve({ success: suggestion.key ? true : undefined, value: suggestion.key });
            }).catch(() => {
                if (timestamp !== this.promiseTimestamp) {
                    reject();
                    return;
                }
                formData = utils_1.updateSafelyWithJSONPointer(formData, undefined, pairCountField);
                this.props.onChange(formData);
                resolve({ success: false, value: value });
            });
        });
    }
};
UnitCountShorthandField.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            shorthandField: PropTypes.string.isRequired,
            pairCountField: PropTypes.string.isRequired,
            taxonIDField: PropTypes.string.isRequired
        }).isRequired
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["object"])
    }).isRequired,
    formData: PropTypes.object.isRequired
};
UnitCountShorthandField = __decorate([
    VirtualSchemaField_1.default
], UnitCountShorthandField);
exports.default = UnitCountShorthandField;
let CodeReader = class CodeReader extends React.Component {
    constructor(props) {
        super(props);
        this.onChange = ({ target: { value } }) => {
            this.props.onChange(value);
        };
        this.onKeyDown = (e) => {
            if (e.key === "Enter") {
                this.onBlur();
            }
        };
        this.state = { loading: false };
        this.onBlur = this.onBlur.bind(this);
    }
    componentDidUpdate(prevProps) {
        if (prevProps.options.taxonID !== this.props.options.taxonID) {
            this.onBlur();
        }
    }
    render() {
        let validationState = null;
        if (this.state.success === false) {
            validationState = "error";
        }
        else if (this.state.success) {
            validationState = "success";
        }
        const { formContext } = this.props;
        const { translations } = formContext;
        const inputElem = (React.createElement(components_1.FetcherInput, { id: this.props.id, loading: this.state.loading, value: this.props.value || "", validationState: validationState, onBlur: this.onBlur, onKeyDown: this.onKeyDown, onChange: this.onChange }));
        const { FormGroup, HelpBlock } = this.context.theme;
        return (React.createElement(FormGroup, { validationState: this.state.success === false ? "error" : undefined },
            inputElem,
            this.state.success === false ? (React.createElement(HelpBlock, null, translations.InvalidUnitCode)) : null,
            React.createElement("div", { className: "small text-muted", dangerouslySetInnerHTML: { __html: this.props.help } })));
    }
    onBlur() {
        const { value, options } = this.props;
        const { parseCode, taxonID } = options;
        if (!(value === this.state.value && taxonID === this.state.taxonID)) {
            this.setState({ value, taxonID, loading: true, success: undefined });
            this.props.formContext.services.submitHooks.add(this.props, () => {
                return parseCode(value, taxonID).then((result) => {
                    this.setState({ loading: false, success: result.success, value: result.value });
                });
            });
        }
    }
};
CodeReader.contextType = ReactContext_1.default;
CodeReader.propTypes = {
    options: PropTypes.shape({
        parseCode: PropTypes.func.isRequired,
        taxonID: PropTypes.string
    }).isRequired,
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["string"])
    }),
    value: PropTypes.string
};
CodeReader = __decorate([
    BaseComponent_1.default
], CodeReader);
