"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const BaseComponent_1 = require("../BaseComponent");
const utils_1 = require("../../utils");
const components_1 = require("../components");
const ReactContext_1 = require("../../ReactContext");
const TagArrayField_1 = require("./TagArrayField");
/**
 * Compatible only with unit array.
 */
let UnitListShorthandArrayField = class UnitListShorthandArrayField extends React.Component {
    constructor() {
        super(...arguments);
        this.onButtonClick = () => () => {
            this.setState({ show: !this.state.show });
        };
        this.onHide = () => {
            this.setState({ show: false, value: "" });
        };
        this.onTagFieldChange = (tags) => {
            this.setState({ value: tags.join(",") });
        };
        this.onSubmit = () => {
            const value = this.state.value + (utils_1.isEmptyString(this.tagRef.state.value) ? "" : `,${this.tagRef.state.value}`);
            this.onHide();
            const { translations, notifier, apiClient } = this.props.formContext;
            this.props.formContext.services.blocker.push();
            apiClient.fetch("/autocomplete/unit", { q: value, list: true, includePayload: true }).then(({ payload: { units, nonMatchingCount } }) => {
                units = units.map(unit => {
                    unit = utils_1.getDefaultFormState(this.props.schema.items, unit);
                    unit = utils_1.bringRemoteFormData(unit, this.props.formContext);
                    unit = utils_1.assignUUID(unit);
                    return unit;
                });
                const formData = this.props.formData;
                const last = formData[formData.length - 1];
                if (last && utils_1.isDefaultData(last, this.props.schema.items)) {
                    formData.pop();
                }
                this.props.onChange([...formData, ...units]);
                nonMatchingCount
                    ? notifier.warning(`${translations.UnitListShorthandWarning} ${nonMatchingCount}`)
                    : notifier.success(translations.UnitListShorthandSuccess);
                this.props.formContext.services.blocker.pop();
            });
        };
        this.onKeyDown = (e) => {
            if (e.key === "Enter") {
                this.onSubmit(e);
            }
        };
        this.setTagArrayRef = (ref) => {
            this.tagRef = ref;
        };
    }
    getStateFromProps(props) {
        const buttonDefinition = {
            fn: this.onButtonClick,
            fnName: "addUnitList",
            glyph: "align-justify",
            label: this.props.formContext.translations.AddUnitList,
            id: this.props.idSchema.$id,
            changesFormData: true,
            rules: {
                canAdd: true
            }
        };
        const innerUiSchema = utils_1.getInnerUiSchema(props.uiSchema);
        const options = utils_1.getUiOptions(innerUiSchema);
        const uiSchema = Object.assign(Object.assign({}, innerUiSchema), { "ui:options": Object.assign(Object.assign({}, options), { buttons: [
                    ...(options.buttons || []),
                    buttonDefinition
                ] }) });
        return { uiSchema };
    }
    render() {
        const { SchemaField } = this.props.registry.fields;
        const { Modal } = this.context.theme;
        return (React.createElement(React.Fragment, null,
            React.createElement(SchemaField, Object.assign({}, this.props, { uiSchema: this.state.uiSchema })),
            this.state.show && (React.createElement(Modal, { show: true, onHide: this.onHide, dialogClassName: "unit-list-shorthand-modal" },
                React.createElement(Modal.Body, null,
                    React.createElement(TagArrayField_1.TagInputComponent, { tags: utils_1.isEmptyString(this.state.value) ? [] : this.state.value.split(",").filter(s => !utils_1.isEmptyString(s)), onChange: this.onTagFieldChange, schema: {}, idSchema: {}, uiSchema: { "ui:options": { separatorKeys: [","] } }, inputProps: { autoFocus: true }, onKeyDown: this.onKeyDown, ref: this.setTagArrayRef, formContext: this.props.formContext }),
                    React.createElement("span", { className: "text-muted" },
                        React.createElement("i", null, this.props.formContext.translations.UnitListShorthandHelp)),
                    React.createElement("br", null),
                    React.createElement(components_1.Button, { type: "submit", disabled: utils_1.isEmptyString(this.state.value), onClick: this.onSubmit }, this.props.formContext.translations.Add))))));
    }
};
UnitListShorthandArrayField.contextType = ReactContext_1.default;
UnitListShorthandArrayField.propTypes = {
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["array"])
    }).isRequired,
    formData: PropTypes.array.isRequired
};
UnitListShorthandArrayField = __decorate([
    BaseComponent_1.default
], UnitListShorthandArrayField);
exports.default = UnitListShorthandArrayField;
