"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getObjectTemplateClassNames = exports.getGlyphButtons = void 0;
const React = require("react");
const utils_1 = require("@rjsf/utils");
const utils_2 = require("../../utils");
const ArrayFieldTemplate_1 = require("./ArrayFieldTemplate");
const ReactContext_1 = require("../../ReactContext");
function ObjectFieldTemplate(props) {
    const TitleFieldTemplate = utils_1.getTemplate("TitleFieldTemplate", props.registry, utils_2.getUiOptions(props.uiSchema));
    const DescriptionFieldTemplate = utils_1.getTemplate("DescriptionFieldTemplate", props.registry, utils_2.getUiOptions(props.uiSchema));
    const { ButtonToolbar } = React.useContext(ReactContext_1.default).theme;
    let buttons = getGlyphButtons(props);
    const [topButtons, bottomButtons, leftButtons, rightButtons] = ["top", "bottom", "left", "right"].map(pos => {
        const buttons = ArrayFieldTemplate_1.getButtonsForPosition(props, utils_2.getUiOptions(props.uiSchema).buttons, pos, "no default");
        return buttons
            ? (React.createElement(ButtonToolbar, { key: `buttons-${pos}` }, buttons))
            : null;
    });
    const { containerClassName, schemaClassName, buttonsClassName } = getObjectTemplateClassNames(props, buttons);
    buttons = React.createElement("div", { className: buttonsClassName }, buttons);
    const titleUiSchema = Object.assign(Object.assign({}, (props.uiSchema || {})), { "_ui:renderedButtons": buttons });
    return (React.createElement("div", { className: containerClassName },
        React.createElement("fieldset", { className: schemaClassName },
            props.title &&
                React.createElement(TitleFieldTemplate, { id: `${props.idSchema.$id}__title`, title: props.title, required: props.required || props.uiSchema["ui:required"], schema: props.schema, uiSchema: titleUiSchema, registry: props.registry }),
            topButtons,
            props.description &&
                React.createElement(DescriptionFieldTemplate, { id: `${props.idSchema.$id}__description`, description: props.description, registry: props.registry, schema: props.schema }),
            leftButtons && (React.createElement("div", { className: "pull-left" }, leftButtons)),
            props.properties.map(({ content }) => content),
            rightButtons && (React.createElement("div", { className: "pull-right" }, rightButtons)),
            bottomButtons),
        !props.title && buttons ? buttons : undefined));
}
exports.default = ObjectFieldTemplate;
function getGlyphButtons(props) {
    const { uiSchema } = props;
    let buttons = uiSchema["ui:buttons"] || [];
    const buttonDescriptions = (utils_2.getUiOptions(uiSchema).buttons || []).filter(buttonDef => !buttonDef.position);
    if (buttonDescriptions) {
        buttons = [
            ...buttons,
            ...buttonDescriptions.map(buttonDescription => ArrayFieldTemplate_1.getButton(buttonDescription, props))
        ];
    }
    return buttons
        ? props.title
            ? buttons.reverse()
            : buttons
        : null;
}
exports.getGlyphButtons = getGlyphButtons;
function getObjectTemplateClassNames(props, buttons) {
    const vertical = props.uiSchema["ui:buttonsVertical"];
    let containerClassName, schemaClassName, buttonsClassName;
    if (buttons && buttons.length) {
        containerClassName = "laji-form-field-template-item" + (vertical ? " keep-vertical" : "");
        schemaClassName = "laji-form-field-template-schema";
        buttonsClassName = "laji-form-field-template-buttons";
        if (props.title)
            buttonsClassName += " pull-right";
    }
    return { containerClassName, schemaClassName, buttonsClassName };
}
exports.getObjectTemplateClassNames = getObjectTemplateClassNames;
