"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const components_1 = require("../components");
const utils_1 = require("../../utils");
const Context_1 = require("../../Context");
const ReactContext_1 = require("../../ReactContext");
const TitleField = ({ title, id, formData, style, uiSchema = {}, registry = {} }) => {
    const { "ui:help": help, "ui:helpHoverable": helpHoverable, "_ui:renderedButtons": buttons } = uiSchema;
    const { titleClassName: className, titleFormatters = [] } = utils_1.getUiOptions(uiSchema);
    const renderedFormatters = titleFormatters.map((titleFormatter) => {
        const { renderer } = titleFormatter;
        return _titleFormatters[renderer](Object.assign(Object.assign({}, titleFormatter), { formData }));
    }).filter(i => i);
    const { Tooltip } = React.useContext(ReactContext_1.default).theme;
    if (renderedFormatters.length === 0 && utils_1.isEmptyString(title)) {
        return null;
    }
    const helpComponent = help ? React.createElement(components_1.Help, null) : null;
    let titleTextContent = (React.createElement(React.Fragment, null,
        React.createElement("span", { dangerouslySetInnerHTML: { __html: title } }),
        " ",
        helpComponent));
    if (help) {
        const tooltipElem = (React.createElement(Tooltip, { id: id + "-tooltip" },
            React.createElement("span", null,
                React.createElement("strong", { dangerouslySetInnerHTML: { __html: title } }),
                React.createElement("br", null),
                React.createElement("span", { dangerouslySetInnerHTML: { __html: help } }))));
        titleTextContent = (React.createElement(components_1.OverlayTrigger, { placement: "right", overlay: tooltipElem, hoverable: helpHoverable, formContext: registry.formContext }, titleTextContent));
    }
    return (React.createElement("legend", { className: utils_1.classNames(className, help && "has-help"), style: style },
        React.createElement("span", null,
            React.createElement("span", null, renderedFormatters),
            " ",
            titleTextContent,
            " ",
            buttons)));
};
exports.default = TitleField;
const _titleFormatters = {
    informalTaxonGroup: ({ formData, value, renderer }) => {
        const informalTaxonGroup = utils_1.parseJSONPointer(formData, value, !!"safely");
        const { informalTaxonGroupsById = {} } = Context_1.default();
        const name = informalTaxonGroupsById[informalTaxonGroup] ? informalTaxonGroupsById[informalTaxonGroup].name : "";
        return informalTaxonGroup ? React.createElement("span", { key: renderer },
            React.createElement("div", { className: `informal-group-image ${informalTaxonGroup}` }),
            " ",
            name) : undefined;
    }
};
