"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_dom_1 = require("react-dom");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const ReactContext_1 = require("../../ReactContext");
class CheckboxWidget extends React.Component {
    constructor(props) {
        super(props);
        this.rmInputTabIndices = () => {
            [this.trueRef, this.falseRef, this.undefinedRef].forEach(node => {
                const domNode = react_dom_1.findDOMNode(node.current);
                if (!domNode) {
                    return;
                }
                const input = domNode.getElementsByTagName("input")[0];
                if (!input) {
                    return;
                }
                input.setAttribute("tabindex", -1);
            });
        };
        this.onChange = (value) => {
            if (value !== undefined && this.getOptions(this.props).invert) {
                value = !value;
            }
            this.props.onChange(value);
        };
        this.onButtonGroupChange = (value) => {
            if (value === "undefined") {
                value = undefined;
            }
            this.onChange(value);
        };
        this.getOptions = (props) => {
            const { Yes, No, Unknown } = props.registry.formContext.translations;
            return Object.assign({ allowUndefined: true, showUndefined: true, invert: false, trueLabel: Yes, falseLabel: No, unknownLabel: Unknown, required: this.props.required }, utils_1.getUiOptions(props));
        };
        this.getToggleMode = (props) => {
            const { allowUndefined, showUndefined, falseLabel, trueLabel } = this.getOptions(props);
            const displayUndefined = (allowUndefined && showUndefined);
            const { Yes, No } = props.registry.formContext.translations;
            return !displayUndefined && (trueLabel === Yes && falseLabel === No);
        };
        this.onGroupKeyDown = this.props.formContext.utils.keyboardClick((e) => {
            this.getToggleMode(this.props) && this.toggle(e);
        });
        this.onTrueKeyDown = this.props.formContext.utils.keyboardClick(() => {
            if (this.props.disabled || this.props.readonly) {
                return;
            }
            this.onChange(true);
        });
        this.onFalseKeyDown = this.props.formContext.utils.keyboardClick(() => {
            if (this.props.disabled || this.props.readonly) {
                return;
            }
            this.onChange(false);
        });
        this.onUndefinedKeyDown = this.props.formContext.utils.keyboardClick(() => {
            this.onChange(undefined);
        });
        this.toggle = (e) => {
            if (this.props.disabled || this.props.readonly) {
                return;
            }
            const nodes = [this.trueRef, this.falseRef, this.groupRef].map(r => react_dom_1.findDOMNode(r.current));
            if (!nodes.includes(e.target)) {
                return;
            }
            e.preventDefault();
            this.props.onChange(!this.props.value);
        };
        this.trueRef = React.createRef();
        this.falseRef = React.createRef();
        this.undefinedRef = React.createRef();
        this.groupRef = React.createRef();
    }
    componentDidMount() {
        this.rmInputTabIndices();
    }
    componentDidUpdate() {
        this.rmInputTabIndices();
    }
    render() {
        const { value, disabled, readonly, label, id } = this.props;
        const options = this.getOptions(this.props);
        const { allowUndefined, showUndefined, invert, trueLabel, falseLabel, unknownLabel, required, label: uiOptionsLabel } = options;
        const hasLabel = !utils_1.isEmptyString(label) && uiOptionsLabel !== false;
        // "undefined" for silencing ToggleButton warning.
        const _value = value === undefined
            ? "undefined"
            : invert
                ? !value
                : value;
        const { ButtonToolbar, ToggleButton, ToggleButtonGroup } = this.context.theme;
        const displayUndefined = (allowUndefined && showUndefined);
        const toggleMode = this.getToggleMode(this.props);
        const _disabled = disabled || readonly;
        const commonProps = {
            disabled: _disabled,
            tabIndex: (toggleMode || _disabled) ? undefined : 0
        };
        const tabTargetClass = "laji-form-checkbox-widget-tab-target";
        const checkbox = (React.createElement(ButtonToolbar, { className: utils_1.classNames(toggleMode && "desktop-layout") },
            React.createElement(ToggleButtonGroup, Object.assign({ ref: this.groupRef, type: "radio", value: [_value], name: this.props.id, onChange: this.onButtonGroupChange, onKeyDown: this.onGroupKeyDown, className: utils_1.classNames(toggleMode && tabTargetClass) }, commonProps, { tabIndex: (toggleMode && !_disabled) ? 0 : undefined }),
                React.createElement(ToggleButton, Object.assign({ id: `${id}-true`, ref: this.trueRef, value: true, onClick: toggleMode ? this.toggle : undefined, className: utils_1.classNames(toggleMode && _value === false && "laji-form-hide-btn-label", !toggleMode && tabTargetClass), onKeyDown: this.onTrueKeyDown }, commonProps), trueLabel),
                React.createElement(ToggleButton, Object.assign({ id: `${id}-false`, ref: this.falseRef, value: false, onClick: toggleMode ? this.toggle : undefined, className: utils_1.classNames(toggleMode && _value === true && "laji-form-hide-btn-label"), onKeyDown: this.onFalseKeyDown }, commonProps), falseLabel),
                (displayUndefined ? React.createElement(ToggleButton, Object.assign({ id: `${id}-undefined`, ref: this.undefinedRef, value: "undefined" }, commonProps, { onKeyDown: this.onUndefinedKeyDown }), unknownLabel) : null))));
        const { Label } = this.props.formContext;
        return !hasLabel ? checkbox : (React.createElement(Label, { label: label, required: required, uiSchema: options, contextId: this.props.formContext.contextId }, checkbox));
    }
    formatValue(value, options, props) {
        return value === undefined
            ? ""
            : value === true
                ? props.formContext.translations.Yes
                : props.formContext.translations.No;
    }
}
exports.default = CheckboxWidget;
CheckboxWidget.contextType = ReactContext_1.default;
CheckboxWidget.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            allowUndefined: PropTypes.bool,
            showUndefined: PropTypes.bool,
            invert: PropTypes.bool,
            trueLabel: PropTypes.string,
            falseLabel: PropTypes.string,
            unknownLabel: PropTypes.string
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["boolean"])
    }),
    value: PropTypes.bool
};
