"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const moment = require("moment");
const DateTimeWidget_1 = require("./DateTimeWidget");
const utils_1 = require("../../utils");
const format = (allowOnlyYear, value) => moment(value).format(allowOnlyYear && value.match(DateTimeWidget_1.YEAR_MATCH)
    ? "YYYY"
    : "YYYY-MM-DD");
const DateWidget = (props) => {
    const { onChange } = props;
    const { allowOnlyYear } = utils_1.getUiOptions(props);
    const _onChange = React.useCallback((value) => onChange(utils_1.isEmptyString(value) ? undefined : format(allowOnlyYear, value)), [allowOnlyYear, onChange]);
    return (React.createElement(DateTimeWidget_1.default, Object.assign({}, props, { onChange: _onChange, time: false, value: props.value ? format(allowOnlyYear, props.value) : null })));
};
DateWidget.propTypes = {
    uiSchema: PropTypes.shape({
        "ui:options": PropTypes.shape({
            showButtons: PropTypes.oneOfType([
                PropTypes.bool,
                PropTypes.shape({
                    today: PropTypes.bool,
                    yesterday: PropTypes.bool,
                    same: PropTypes.oneOfType([
                        PropTypes.bool,
                        PropTypes.shape({
                            path: PropTypes.string
                        })
                    ])
                })
            ]),
            allowOnlyYear: PropTypes.bool
        })
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["string"])
    }),
    value: PropTypes.string
};
exports.default = DateWidget;
