"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const immutability_helper_1 = require("immutability-helper");
const BaseInputTemplate_1 = require("../templates/BaseInputTemplate");
const ReactContext_1 = require("../../ReactContext");
class TextSelectWidget extends React.Component {
    constructor(props) {
        super(props);
        this.onCheckBoxChange = (label) => () => {
            const selectedCheckboxes = this.state.selectedCheckboxes;
            if (selectedCheckboxes.has(label)) {
                selectedCheckboxes.delete(label);
            }
            else {
                selectedCheckboxes.add(label);
            }
            this.setState({ selectedCheckboxes: selectedCheckboxes });
            this.onChange(this.state.otherValue);
        };
        this.onInputChange = (value) => {
            this.onChange(value);
        };
        this.onChange = (textValue) => {
            let newValue = [...Array.from(this.state.selectedCheckboxes)];
            if (this.state.selectedCheckboxes.has(this.state.freeTextField)) {
                if (textValue && textValue.length > 0) {
                    newValue = immutability_helper_1.default(newValue, { $push: [textValue] });
                }
                newValue.splice(newValue.indexOf(this.state.freeTextField), 1);
            }
            this.props.onChange(newValue.join(", "));
        };
        this.state = this.getInitialState(props);
    }
    UNSAFE_componentWillReceiveProps(props) {
        this.setState(this.getStateFromProps(props));
    }
    getInitialState(props) {
        let { enums = [], delimiter = ", ", freeTextField } = utils_1.getUiOptions(props);
        const { value } = props;
        const selectedCheckboxes = new Set();
        const values = !value ? [] : value.split(delimiter);
        const otherValues = [];
        values.map(val => {
            if (enums.indexOf(val) !== -1) {
                selectedCheckboxes.add(val);
            }
            else {
                otherValues.push(val);
            }
        });
        if (otherValues.length > 0) {
            selectedCheckboxes.add(freeTextField);
        }
        return { selectedCheckboxes, enums, otherValue: otherValues.join(delimiter), freeTextField };
    }
    getStateFromProps(props) {
        let { enums = [], delimiter = ", " } = utils_1.getUiOptions(props);
        const { value } = props;
        const values = !value ? [] : value.split(delimiter);
        const otherValues = [];
        values.map(val => {
            if (enums.indexOf(val) === -1) {
                otherValues.push(val);
            }
        });
        return { otherValue: otherValues.join(delimiter) };
    }
    render() {
        let { enums, otherValue, freeTextField, selectedCheckboxes } = this.state;
        if (freeTextField) {
            enums = immutability_helper_1.default(enums, { $push: [freeTextField] });
        }
        const { Checkbox } = this.context.theme;
        return (React.createElement("div", { className: "laji-text-select" },
            enums.map((label, i) => {
                return (React.createElement(Checkbox, { key: i, title: label, checked: selectedCheckboxes.has(label), onChange: this.onCheckBoxChange(label) }, label));
            }),
            freeTextField ?
                React.createElement(BaseInputTemplate_1.default, Object.assign({}, this.props, { id: this.props.id + "_input", value: otherValue, onChange: this.onInputChange, disabled: !selectedCheckboxes.has(freeTextField) }))
                : null));
    }
}
TextSelectWidget.contextType = ReactContext_1.default;
TextSelectWidget.propTypes = {
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["string"])
    }).isRequired,
    value: PropTypes.string
};
exports.default = TextSelectWidget;
