"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const PropTypes = require("prop-types");
const utils_1 = require("../../utils");
const components_1 = require("../components");
const Context_1 = require("../../Context");
const ReactContext_1 = require("../../ReactContext");
const utils_2 = require("../../utils");
class TextareaWidget extends React.Component {
    constructor(props) {
        super(props);
        this.getStateFromProps = (props) => {
            return { value: props.value };
        };
        this.onFocus = (e) => {
            this.focused = true;
            this.props.onFocus && this.props.onFocus(e);
        };
        this.onBlur = (e) => {
            this.focused = false;
            if (this.state.value !== this.props.value) {
                this.props.onChange(this.state.value);
            }
            if (this.timeout)
                clearTimeout(this.timeout);
            this.props.onBlur && this.props.onBlur(e);
        };
        this.onChange = (value) => {
            this.setState({ value }, () => {
                if (!this.focused) {
                    this.props.onChange(value);
                }
                else {
                    if (this.timeout)
                        clearTimeout(this.timeout);
                    this.timeout = this.props.formContext.setTimeout(() => {
                        this.props.onChange(value === "" ? utils_2.getUiOptions(this.props).emptyValue : value);
                    }, 1000);
                }
            });
        };
        this._onChange = ({ target: { value } }) => {
            this.onChange(value);
        };
        this.onKeyDown = (e) => {
            const inputAllowed = !this.props.disabled && !this.props.readonly;
            const isCtrlEnter = e.ctrlKey && e.key === "Enter";
            if (this.state.enterReserved && inputAllowed && isCtrlEnter) {
                this.onChange(this.state.value + "\n");
                e.preventDefault();
                e.stopPropagation();
            }
        };
        this.state = this.getStateFromProps(props);
        this.textareaRef = React.createRef();
        this._context = Context_1.default(props.formContext.contextId);
        Object.keys(props.formContext.services.keyHandler.shortcuts).some(keyCombo => {
            if (keyCombo === "Enter") {
                //	// Direct mutation should be ok in constructor.
                this.state.enterReserved = true; // eslint-disable-line react/no-direct-mutation-state
            }
        });
    }
    UNSAFE_componentWillReceiveProps(props) {
        this.setState(this.getStateFromProps(props));
    }
    render() {
        const { id, options, placeholder, disabled, readonly, autofocus } = this.props;
        const { value } = this.state;
        const { required = this.props.required } = options;
        const textarea = React.createElement("textarea", { id: id, className: "form-control", value: typeof value === "undefined" ? "" : value, placeholder: placeholder, required: required, disabled: disabled, readOnly: readonly, autoFocus: autofocus, rows: options.rows, onFocus: this.onFocus, onBlur: this.onBlur, onChange: this._onChange, onKeyDown: this.onKeyDown });
        return this.state.enterReserved ? (React.createElement(components_1.TooltipComponent, { tooltip: `${utils_1.stringifyKeyCombo("ctrl+Enter")} ${this.props.formContext.translations.textareaHint}`, placement: "bottom" }, textarea)) : textarea;
    }
}
exports.default = TextareaWidget;
TextareaWidget.propTypes = {
    "ui:options": PropTypes.shape({
        emptyValue: PropTypes.string,
        rows: PropTypes.number,
    }),
    schema: PropTypes.shape({
        type: PropTypes.oneOf(["string"])
    }),
    value: PropTypes.string
};
TextareaWidget.contextType = ReactContext_1.default;
