"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const moment = require("moment");
describe("Date & time widgets", () => {
    let form;
    let widget;
    const schema = {
        type: "string"
    };
    const formData = null;
    const today = moment();
    const today0100 = moment(`${moment().format("YYYY-MM-DD")}T01:00`);
    const yesterday = moment().add(-1, "days");
    const ISO8601DateFormat = "YYYY-MM-DD";
    const ISO8601TimeFormat = "HH:mm";
    const ISO8601FullFormat = `${ISO8601DateFormat}T${ISO8601TimeFormat}`;
    const displayDateFormat = "DD.MM.YYYY";
    const displayTimeFormat = "HH.mm";
    const displayFullFormat = `${displayDateFormat}, ${displayTimeFormat}`;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm();
        widget = form.getDateWidget("");
    }));
    describe("DateTimeWidget", () => {
        const uiSchema = {
            "ui:widget": "DateTimeWidget",
        };
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ schema, uiSchema, formData });
        }));
        it("renders date button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.isDisplayed(widget.buttons.$date)).toBe(true);
        }));
        it("renders time button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.isDisplayed(widget.buttons.$time)).toBe(true);
        }));
        it("selecting date from calendar adds only date", () => __awaiter(void 0, void 0, void 0, function* () {
            yield widget.buttons.$date.click();
            yield widget.calendar.waitAnimation();
            yield widget.calendar.$today.click();
            expect(yield widget.$input.getAttribute("value")).toBe(today.format(displayDateFormat));
            expect(yield form.getChangedData()).toBe(today.format(ISO8601DateFormat));
        }));
        it("selecting time adds time", () => __awaiter(void 0, void 0, void 0, function* () {
            yield widget.buttons.$time.click();
            yield widget.clock.waitAnimation();
            yield widget.clock["$01:00"].click();
            expect(yield widget.$input.getAttribute("value")).toBe(today0100.format(displayFullFormat));
            expect(yield form.getChangedData()).toBe(today0100.format(ISO8601FullFormat));
        }));
        it("time can be typed", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, `${today.format(displayDateFormat)}, 10`);
            expect(yield widget.$input.getAttribute("value")).toBe(`${today.format(displayDateFormat)}, 10.00`);
            expect(yield form.getChangedData()).toBe(`${today.format(ISO8601DateFormat)}T10:00`);
        }));
        it("time can be removed by clearing it from input", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, today.format(displayDateFormat));
            expect(yield widget.$input.getAttribute("value")).toBe(today.format(displayDateFormat));
            expect(yield form.getChangedData()).toBe(today.format(ISO8601DateFormat));
        }));
        it("whole value can be cleared", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, "");
            expect(yield widget.$input.getAttribute("value")).toBe("");
            expect(yield form.getChangedData()).toBe(null);
        }));
        it("date can be typed", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, "2.4.2012");
            expect(yield widget.$input.getAttribute("value")).toBe("02.04.2012");
            expect(yield form.getChangedData()).toBe("2012-04-02");
        }));
        describe("today & yesterday buttons", () => {
            it("not displayed by default", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield test_utils_1.isDisplayed(widget.buttons.$today)).toBe(false);
                expect(yield test_utils_1.isDisplayed(widget.buttons.$yesterday)).toBe(false);
            }));
            it("displayed if showButtons true", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ uiSchema: Object.assign(Object.assign({}, uiSchema), { "ui:options": { showButtons: true } }) });
                expect(yield test_utils_1.isDisplayed(widget.buttons.$today)).toBe(true);
                expect(yield test_utils_1.isDisplayed(widget.buttons.$yesterday)).toBe(true);
            }));
            it("yesterday works", () => __awaiter(void 0, void 0, void 0, function* () {
                yield widget.buttons.$yesterday.click();
                expect(yield widget.$input.getAttribute("value")).toBe(yesterday.format(displayDateFormat));
                expect(yield form.getChangedData()).toBe(yesterday.format(ISO8601DateFormat));
            }));
            it("today works", () => __awaiter(void 0, void 0, void 0, function* () {
                yield widget.buttons.$today.click();
                expect(yield widget.$input.getAttribute("value")).toBe(today.format(displayDateFormat));
                expect(yield form.getChangedData()).toBe(today.format(ISO8601DateFormat));
            }));
        });
        describe("typing only year when option allowOnlyYear", () => {
            const year = "" + moment().year();
            it("is off should enter full date", () => __awaiter(void 0, void 0, void 0, function* () {
                yield test_utils_1.updateValue(widget.$input, year);
                expect(yield widget.$input.getAttribute("value")).toBe(`01.01.${year}`);
                expect(yield form.getChangedData()).toBe(today.format(`${year}-01-01`));
            }));
            it("is on works", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({ uiSchema: Object.assign(Object.assign({}, uiSchema), { "ui:options": { allowOnlyYear: true } }) });
                yield test_utils_1.updateValue(widget.$input, year);
                expect(yield widget.$input.getAttribute("value")).toBe(year);
                expect(yield form.getChangedData()).toBe(year);
            }));
        });
        describe("same button", () => {
            let startWidget;
            let endWidget;
            beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
                yield form.setState({
                    schema: { type: "object", properties: { start: { type: "string" }, end: { type: "string" } } },
                    uiSchema: {
                        start: Object.assign(Object.assign({}, uiSchema), { "ui:options": { showButtons: true } }),
                        end: Object.assign(Object.assign({}, uiSchema), { "ui:options": { showButtons: { same: { path: "/start" } } } })
                    },
                    formData: {}
                });
                startWidget = form.getDateWidget("start");
                endWidget = form.getDateWidget("end");
            }));
            it("not displayed by if showButtons true", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield test_utils_1.isDisplayed(startWidget.buttons.$same)).toBe(false);
            }));
            it("displayed if showButtons has {same: true}", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield test_utils_1.isDisplayed(endWidget.buttons.$same)).toBe(true);
            }));
            it("click works", () => __awaiter(void 0, void 0, void 0, function* () {
                yield test_utils_1.updateValue(startWidget.$input, "2.4.2012");
                yield endWidget.buttons.$same.click();
                expect(yield endWidget.$input.getAttribute("value")).toBe("02.04.2012");
                expect((yield form.getChangedData()).end).toBe("2012-04-02");
            }));
        });
    });
    describe("DateWidget", () => {
        const uiSchema = {
            "ui:widget": "DateWidget",
        };
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ schema, uiSchema, formData });
        }));
        it("renders date button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.isDisplayed(widget.buttons.$date)).toBe(true);
        }));
        it("doesn't render time button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.isDisplayed(widget.buttons.$time)).not.toBe(true);
        }));
        it("clears time if typed", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, `${today.format(displayDateFormat)}, 10.00`);
            expect(yield widget.$input.getAttribute("value")).toBe(today.format(displayDateFormat));
            expect(yield form.getChangedData()).toBe(today.format(ISO8601DateFormat));
        }));
    });
    describe("TimeWidget", () => {
        const uiSchema = {
            "ui:widget": "TimeWidget",
        };
        beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, "");
            yield form.setState({ schema, uiSchema });
        }));
        it("renders time button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.isDisplayed(widget.buttons.$time)).toBe(true);
        }));
        it("doesn't render date button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.isDisplayed(widget.buttons.$date)).not.toBe(true);
        }));
        it("time can be typed", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.updateValue(widget.$input, "10.00");
            expect(yield widget.$input.getAttribute("value")).toBe("10.00");
            expect(yield form.getChangedData()).toBe("10:00");
        }));
        it("showTimeList hides time button", () => __awaiter(void 0, void 0, void 0, function* () {
            yield form.setState({ uiSchema: Object.assign(Object.assign({}, uiSchema), { "ui:options": { showTimeList: false } }) });
            expect(yield test_utils_1.isDisplayed(widget.buttons.$time)).toBe(false);
        }));
    });
});
