"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("Geocoder", () => {
    let form;
    const schema = {
        type: "object",
        properties: {
            country: {
                type: "string"
            },
            municipality: {
                type: "string"
            },
            biologicalProvince: {
                type: "string"
            },
            geometry: {
                type: "object",
                properties: {
                    coordinates: {
                        type: "array",
                        items: {
                            type: "number"
                        }
                    },
                    type: {
                        type: "string"
                    }
                }
            }
        }
    };
    const uiSchema = {
        "ui:field": "GeocoderField"
    };
    const formData = { geometry: { type: "Point", coordinates: [25, 60] } };
    const response = {
        "status": "OK",
        "results": [
            {
                "address_components": [
                    {
                        "long_name": "Nylandia",
                        "short_name": "N",
                        "types": [
                            "biogeographicalProvince"
                        ]
                    }
                ],
                "types": [
                    "biogeographicalProvince"
                ]
            },
            {
                "address_components": [
                    {
                        "short_name": "Helsinki",
                        "types": [
                            "municipality"
                        ]
                    }
                ],
                "formatted_address": "Helsinki",
                "types": [
                    "municipality"
                ]
            }
        ]
    };
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm();
    }));
    it("parses location in Finland correctly", () => __awaiter(void 0, void 0, void 0, function* () {
        const { resolve, remove } = yield form.setMockResponse("/coordinates/location", false);
        yield form.setState({ schema, uiSchema, formData });
        expect(yield form.$locate("country").$("input").getAttribute("value")).toBe("");
        expect(yield form.$locate("municipality").$("input").getAttribute("value")).toBe("");
        expect(yield form.$locate("biologicalProvince").$("input").getAttribute("value")).toBe("");
        yield resolve(response);
        yield remove();
        expect(yield form.$locate("country").$("input").getAttribute("value")).toBe("Suomi");
        expect(yield form.$locate("municipality").$("input").getAttribute("value")).toBe("Helsinki");
        expect(yield form.$locate("biologicalProvince").$("input").getAttribute("value")).toBe("Nylandia");
    }));
    const $runningJobs = protractor_1.$(".running-jobs");
    it("blocks submit until done", () => __awaiter(void 0, void 0, void 0, function* () {
        const { resolve, remove } = yield form.setMockResponse("/coordinates/location", false);
        yield form.setState({ schema, uiSchema, formData });
        yield form.e("submit()");
        expect(yield $runningJobs.isDisplayed()).toBe(true);
        yield resolve(response);
        yield remove();
        expect(yield $runningJobs.isPresent()).toBe(false);
        expect(yield form.$locate("country").$("input").getAttribute("value")).toBe("Suomi");
        expect(yield form.getSubmittedData()).not.toBe(null);
    }));
    describe("rejecting", () => {
        it("blocks submit", () => __awaiter(void 0, void 0, void 0, function* () {
            const { reject, remove } = yield form.setMockResponse("/coordinates/location", false);
            yield form.setState({ schema, uiSchema, formData });
            yield form.setState({ formData: {} });
            yield form.setState({ formData });
            yield form.e("submit()");
            expect(yield form.getSubmittedData()).toBe(null);
            expect(yield $runningJobs.isDisplayed()).toBe(true);
            yield reject();
            yield remove();
            expect(yield protractor_1.$$(".laji-form-failed-jobs-list .list-group-item").count()).toBe(1);
            expect(yield $runningJobs.isPresent()).toBe(false);
            expect(yield form.getSubmittedData()).not.toBe(null);
        }));
        it("and then locating again removes old bg job", () => __awaiter(void 0, void 0, void 0, function* () {
            const { resolve, remove } = yield form.setMockResponse("/coordinates/location", false);
            yield form.setState({ schema, uiSchema, formData });
            yield form.e("submit()");
            expect(yield protractor_1.$$(".laji-form-failed-jobs-list .list-group-item").count()).toBe(0);
            expect(yield $runningJobs.isDisplayed()).toBe(true);
            yield resolve(response);
            yield remove();
            expect(yield protractor_1.$$(".laji-form-failed-jobs-list .list-group-item").count()).toBe(0);
            expect(yield $runningJobs.isPresent()).toBe(false);
        }));
    });
});
