"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("SelectWidget", () => {
    let form;
    const enums = {
        "": "",
        a: "aLabel",
        b: "bLabel",
        c: "cLabel",
    };
    const schema = {
        type: "string",
        enum: Object.keys(enums),
        enumNames: Object.keys(enums).map(e => enums[e])
    };
    const initialFormData = "b";
    let enumWidget;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ schema, formData: initialFormData });
        enumWidget = form.$getEnumWidget("");
    }));
    it("renders initial value", () => __awaiter(void 0, void 0, void 0, function* () {
        expect(yield enumWidget.$input.getAttribute("value")).toBe(enums[initialFormData]);
    }));
    it("empty value is selectable", () => __awaiter(void 0, void 0, void 0, function* () {
        yield enumWidget.openEnums();
        yield enumWidget.$$enums.first().click();
        expect(yield form.getChangedData()).toBe(null);
    }));
    it("nonempty value is selectable after empty was selected", () => __awaiter(void 0, void 0, void 0, function* () {
        yield enumWidget.openEnums();
        yield enumWidget.$$enums.last().click();
        expect(yield enumWidget.$input.getAttribute("value")).toBe(enums.c);
        expect(yield form.getChangedData()).toBe("c");
    }));
    it("selecting value and then empty value without blurring in between keeps empty value", () => __awaiter(void 0, void 0, void 0, function* () {
        yield enumWidget.openEnums();
        yield enumWidget.$$enums.last().click();
        yield enumWidget.openEnums();
        yield enumWidget.$$enums.first().click();
        expect(yield form.getChangedData()).toBe(null);
    }));
    it("value can be selected with enter", () => __awaiter(void 0, void 0, void 0, function* () {
        yield enumWidget.openEnums();
        yield enumWidget.$input.sendKeys(protractor_1.protractor.Key.DOWN);
        yield enumWidget.$input.sendKeys(protractor_1.protractor.Key.ENTER);
        expect(yield enumWidget.$input.getAttribute("value")).toBe(enums.a);
        expect(yield form.getChangedData()).toBe("a");
    }));
    it("value can be selected with tab", () => __awaiter(void 0, void 0, void 0, function* () {
        yield enumWidget.openEnums();
        yield enumWidget.$input.sendKeys(protractor_1.protractor.Key.DOWN);
        yield enumWidget.$input.sendKeys(protractor_1.protractor.Key.TAB);
        expect(yield enumWidget.$input.getAttribute("value")).toBe(enums.b);
        expect(yield form.getChangedData()).toBe("b");
    }));
});
