"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
describe("transaction form (MHL.930)", () => {
    let form;
    let typeEnum$;
    const uiSchemaContext = {
        userName: "Test, User",
        userEmail: "user.test@email.com"
    };
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "MHL.930" });
        yield form.setState({ uiSchemaContext });
        typeEnum$ = form.$getEnumWidget("type");
        yield typeEnum$.openEnums();
        yield typeEnum$.$$enums.first().click();
    }));
    it("sets correct name and email when clicking a me button", () => __awaiter(void 0, void 0, void 0, function* () {
        yield form.$locate("localPerson").$("button").click();
        const $input = form.$locate("localPerson").$("input");
        expect(yield $input.getAttribute("value")).toBe("Test, User");
        const $input2 = form.$locate("localPersonEmail").$("input");
        expect(yield $input2.getAttribute("value")).toBe("user.test@email.com");
        const formData = yield form.getChangedData();
        expect(formData.localPerson).toBe("Test, User");
        expect(formData.localPersonEmail).toBe("user.test@email.com");
    }));
    describe("specimen fields", () => {
        it("adds the ids", () => __awaiter(void 0, void 0, void 0, function* () {
            const $input = form.$getInputWidget("awayIDs");
            yield $input.sendKeys("abc cde efg");
            yield $input.sendKeys(protractor_1.protractor.Key.TAB);
            let formData = yield form.getChangedData();
            expect(formData.awayIDs).toEqual(["abc", "cde", "efg"]);
        }));
        it("marks all as returned", () => __awaiter(void 0, void 0, void 0, function* () {
            const returnAllButton$ = protractor_1.$(".laji-form-multi-tag-array-field-buttons").all(protractor_1.by.tagName("button")).get(0);
            yield returnAllButton$.click();
            const formData = yield form.getChangedData();
            expect(formData.awayIDs).toEqual([]);
            expect(formData.returnedIDs).toEqual(["abc", "cde", "efg"]);
        }));
        it("marks selected ids as missing", () => __awaiter(void 0, void 0, void 0, function* () {
            const markAsMissingButton$ = protractor_1.$(".laji-form-multi-tag-array-field-buttons").all(protractor_1.by.tagName("button")).get(3);
            yield markAsMissingButton$.click();
            const tags$ = form.$locate("returnedIDs").$$(".rw-multiselect-tag");
            yield tags$.get(1).click();
            const formData = yield form.getChangedData();
            expect(formData.returnedIDs).toEqual(["abc", "efg"]);
            expect(formData.missingIDs).toEqual(["cde"]);
        }));
    });
});
