"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("./test-utils");
const protractor_1 = require("protractor");
const { googleApiKey } = require("../properties.json");
const _testWidget = (form) => (path, type) => __awaiter(void 0, void 0, void 0, function* () {
    const parsePointer = (container, path) => {
        const [next, ...remaining] = path.split(".");
        const nextObject = container[next];
        if (remaining.length) {
            return parsePointer(nextObject, remaining.join("."));
        }
        return nextObject;
    };
    expect(yield test_utils_1.lajiFormLocate(path).isDisplayed()).toBe(true);
    const beforeChange = parsePointer((yield form.getChangedData()) || {}, path);
    let widget;
    let $secondOption, $otherOptionThanActive;
    switch (type) {
        case "checkbox":
            widget = form.getBooleanWidget(path);
            yield widget.$nonactive.click();
            break;
        case "enum":
            widget = form.$getEnumWidget(path);
            yield widget.$container.click();
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(widget.$$enums.first()), 300, "enum list didn't appear on widget click");
            $secondOption = widget.$$enums.get(1);
            $otherOptionThanActive = (yield $secondOption.getAttribute("class")).includes("rw-state-selected")
                ? widget.$$enums.first()
                : $secondOption;
            yield $otherOptionThanActive.click();
            break;
        case "date":
            widget = form.getDateWidget(path).$input;
            yield widget.click();
            yield widget.sendKeys("1.1.2019");
            yield widget.sendKeys(protractor_1.protractor.Key.TAB);
            break;
        default:
            widget = form.$getInputWidget(path);
            yield widget.sendKeys("1");
            yield widget.sendKeys(protractor_1.protractor.Key.TAB);
            break;
    }
    expect(beforeChange).not.toEqual(parsePointer(yield form.getChangedData(), path));
});
describe("Trip report (JX.519)", () => {
    let form;
    let testWidget;
    let npMock;
    beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
        form = yield test_utils_1.createForm({ id: "JX.519" });
        testWidget = _testWidget(form);
        npMock = yield form.setMockResponse("/named-places");
    }));
    const $gatheringsMap = test_utils_1.lajiFormLocate("gatherings").$(".laji-map");
    describe("gatheringEvent", () => {
        const $gatheringEvent = test_utils_1.lajiFormLocate("gatheringEvent");
        it("container is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $gatheringEvent.isPresent()).toBe(true);
        }));
        describe("contains", () => {
            it("secureLevel which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield $gatheringEvent.element(test_utils_1.lajiFormLocate("secureLevel").locator()).isDisplayed()).toBe(true);
                yield testWidget("secureLevel", "checkbox");
            }));
            it("gatheringEvent.legPublic which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatheringEvent.legPublic", "checkbox");
            }));
            it("gatheringEvent.dateBegin which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatheringEvent.dateBegin", "date");
            }));
            it("gatheringEvent.dateEnd which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatheringEvent.dateEnd", "date");
            }));
            it("keywords which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield $gatheringEvent.element(test_utils_1.lajiFormLocate("keywords").locator()).isDisplayed()).toBe(true);
                yield testWidget("keywords");
            }));
            // TODO TableField messes up ids!
            describe("observers", () => {
                it("displayed", () => __awaiter(void 0, void 0, void 0, function* () {
                    expect(yield test_utils_1.isDisplayed(form.$locate("gatheringEvent.0.leg"))).toBe(true);
                }));
                it("can be added", () => __awaiter(void 0, void 0, void 0, function* () {
                    yield form.$locateButton("gatheringEvent", "add").click();
                    expect(yield test_utils_1.isDisplayed(form.$locate("gatheringEvent.1.leg"))).toBe(true);
                }));
                it("deleting first shows only the remaining empty", () => __awaiter(void 0, void 0, void 0, function* () {
                    // Should be 1st delete button (row 0 is labels)
                    yield protractor_1.$$(".table-field .row").get(1).$("button").click(); // eslint-disable-line protractor/use-first-last
                    expect(yield test_utils_1.isDisplayed(form.$locate("gatheringEvent.1.leg"))).toBe(false);
                    expect(yield form.$getInputWidget("gatheringEvent.0.leg").getAttribute("value")).toBe("");
                }));
            });
        });
    });
    const $gatheringAdd = protractor_1.$("#root_gatherings-add");
    describe("gatherings", () => {
        it("is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings").isDisplayed()).toBe(true);
        }));
        it("is empty", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0").isPresent()).toBe(false);
        }));
        it("map is present", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $gatheringsMap.isDisplayed()).toBe(true);
        }));
        it("creating pointer on map creates gathering", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.putForeignMarkerToMap();
            expect(yield test_utils_1.lajiFormLocate("gatherings.0").isDisplayed()).toBe(true);
        }));
        describe("geocoding", () => {
            it("starts and finishes after adding gathering", () => __awaiter(void 0, void 0, void 0, function* () {
                if (yield test_utils_1.lajiFormLocate("gatherings.0.country").isPresent()) {
                    return;
                }
                const $loadingGeocoderButton = protractor_1.$(".geocoder-btn .react-spinner");
                if (yield $loadingGeocoderButton.isPresent()) {
                    yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.invisibilityOf($loadingGeocoderButton), 6000, "Geocoding timeout");
                }
                expect(yield $loadingGeocoderButton.isPresent()).toBe(false);
            }));
            if (!googleApiKey) {
                pending("Google API key missing");
            }
            it("adds country which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatherings.0.country");
            }));
            it("adds administrativeProvince which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatherings.0.administrativeProvince");
            }));
            it("adds municipality which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatherings.0.municipality");
            }));
        });
        it("contains locality which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.locality");
        }));
        it("contains localityDescription which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.localityDescription");
        }));
        it("contains weather which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.weather");
        }));
        it("contains notes which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.notes");
        }));
        const $additionalsButton = protractor_1.$("#root_gatherings_0-additionals");
        it("has additional fields button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $additionalsButton.isDisplayed()).toBe(true);
        }));
        describe("taxonCensus", () => {
            it("can be added from additional fields", () => __awaiter(void 0, void 0, void 0, function* () {
                yield $additionalsButton.click();
                const $$additionalListItems = protractor_1.$$(".dropdown.open li a");
                yield $$additionalListItems.last().click();
                expect(yield form.$locate("gatherings.0.taxonCensus").isDisplayed()).toBe(true);
                yield $additionalsButton.click();
                expect(yield $$additionalListItems.isPresent()).toBe(false);
            }));
            it("can add items", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.$locateButton("gatherings.0.taxonCensus", "add").click();
            }));
            it("added item is focused", () => __awaiter(void 0, void 0, void 0, function* () {
                expect(yield form.$getInputWidget("gatherings.0.taxonCensus.0.censusTaxonID").getAttribute("id")).toBe(yield test_utils_1.getFocusedId());
            }));
            it("added item is displayed and can be edited", () => __awaiter(void 0, void 0, void 0, function* () {
                yield testWidget("gatherings.0.taxonCensus.0.censusTaxonID");
                yield testWidget("gatherings.0.taxonCensus.0.taxonCensusType", "enum");
            }));
            it("second added item is focused", () => __awaiter(void 0, void 0, void 0, function* () {
                yield form.$locateButton("gatherings.0.taxonCensus", "add").click();
                expect(yield form.$getInputWidget("gatherings.0.taxonCensus.1.censusTaxonID").getAttribute("id")).toBe(yield test_utils_1.getFocusedId());
            }));
        });
        it("add button works", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $gatheringAdd.click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.1").isDisplayed()).toBe(true);
            yield test_utils_1.waitUntilBlockingLoaderHides();
        }));
        it("map is empty for new gathering", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $gatheringsMap.$(".vector-marker.leaflet-interactive").isPresent()).toBe(false);
        }));
        it("items can be closed", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.$("#root_gatherings_1-panel .laji-form-clickable-panel-header").click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.1.units").isPresent()).toBe(false);
        }));
        it("items can be opened", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.$("#root_gatherings_1-panel .laji-form-clickable-panel-header").click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.1.units").isPresent()).toBe(true);
        }));
        it("items can be deleted", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.$("#root_gatherings_1-delete").click();
            yield protractor_1.$("#root_gatherings_1-delete-confirm-yes").click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.1").isPresent()).toBe(false);
            expect(yield form.$locateAddition("gatherings.1", "header").isPresent()).toBe(false);
        }));
        describe("named place button", () => {
            let $addNPButton;
            const npChooser = new test_utils_1.NamedPlaceChooserPO();
            const npList = npChooser.select;
            let testingInsaneReason = "";
            beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
                $addNPButton = form.$locateButton("gatherings", "addNamedPlace");
                if (!(yield test_utils_1.isDisplayed($addNPButton))) {
                    testingInsaneReason = "Using place failed. Maybe you don't have named places or your user token in properties.json is expired?";
                }
            }));
            it("click displays list", () => __awaiter(void 0, void 0, void 0, function* () {
                if (testingInsaneReason) {
                    return pending(testingInsaneReason);
                }
                yield $addNPButton.click();
                yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(npList.$container), 500, "NP modal didn't show up");
                expect(yield test_utils_1.isDisplayed(npList.$container)).toBe(true);
            }));
            it("selecting from list displays map popup", () => __awaiter(void 0, void 0, void 0, function* () {
                if (testingInsaneReason) {
                    return pending(testingInsaneReason);
                }
                yield npList.openEnums();
                yield npList.$$enums.first().click();
            }));
            it("map popup select btn click closes chooser modal", () => __awaiter(void 0, void 0, void 0, function* () {
                if (testingInsaneReason) {
                    return pending(testingInsaneReason);
                }
                yield npChooser.mapPopup.$useBtn.click();
            }));
            it("np chooser modal closes after np selected or shows alert", () => __awaiter(void 0, void 0, void 0, function* () {
                if (testingInsaneReason) {
                    return pending(testingInsaneReason);
                }
                if (yield test_utils_1.isDisplayed(npChooser.$alert)) {
                    testingInsaneReason = "Using place failed. Have you configured your user token to properties.json?";
                }
                if (testingInsaneReason) {
                    yield npChooser.$close.click();
                    return pending(testingInsaneReason);
                }
                yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.invisibilityOf(npList.$container), 500, "NP modal didn't close");
                expect(yield test_utils_1.isDisplayed(npList.$container)).toBe(false);
            }));
            it("new gathering is added", () => __awaiter(void 0, void 0, void 0, function* () {
                if (testingInsaneReason) {
                    return pending(testingInsaneReason);
                }
                expect(yield test_utils_1.isDisplayed(form.$locate("gatherings.1"))).toBe(true);
                yield form.$locateButton("gatherings.1", "delete").click();
                yield form.$locateButton("gatherings.1", "delete-confirm-yes").click();
            }));
        });
    });
    describe("units", () => {
        const $unitAdd = protractor_1.$("#root_gatherings_0_units-add");
        it("is displayed", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units").isDisplayed()).toBe(true);
        }));
        it("has one by default", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0").isDisplayed()).toBe(true);
        }));
        it("can be added", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $unitAdd.click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.1").isDisplayed()).toBe(true);
        }));
        it("added is focused", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield form.$getInputWidget("gatherings.0.units.1.identifications.0.taxon").getAttribute("id")).toBe(yield test_utils_1.getFocusedId());
        }));
        it("first is shown as table row after activating second", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0").getTagName()).toBe("tr");
        }));
        it("activating works for both", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.lajiFormLocate("gatherings.0.units.0").click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0").getTagName()).toBe("div");
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.1").getTagName()).toBe("tr");
            yield test_utils_1.lajiFormLocate("gatherings.0.units.1").click();
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.1").getTagName()).toBe("div");
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0").getTagName()).toBe("tr");
        }));
        it("can be deleted", () => __awaiter(void 0, void 0, void 0, function* () {
            yield test_utils_1.removeUnit(0, 1);
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.1").isPresent()).toBe(false);
        }));
        it("first is active after deleting second", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0").getTagName()).toBe("div");
        }));
        it("contains identifications.0.taxon", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0.identifications.0.taxon").isDisplayed()).toBe(true);
        }));
        it("contains count which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.units.0.count");
        }));
        it("contains notes which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.units.0.notes");
        }));
        it("contains taxonConfidence which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.units.0.taxonConfidence", "enum");
        }));
        it("contains recordBasis which is editable", () => __awaiter(void 0, void 0, void 0, function* () {
            yield testWidget("gatherings.0.units.0.recordBasis", "enum");
        }));
        it("contains images", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0.images").isDisplayed()).toBe(true);
        }));
        const $additionalsButton = protractor_1.$("#root_gatherings_0_units_0-additionals");
        const $modal = protractor_1.$(".scope-field-modal");
        it("can open additionals chooser", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $additionalsButton.isDisplayed()).toBe(true);
            const informalGroups = [1, 343, 232, 187];
            let mocks = [];
            for (let id of informalGroups) {
                mocks.push(yield form.setMockResponse(`/informal-taxon-groups/MVL.${id}`));
            }
            yield $additionalsButton.click();
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf($modal), 5000, "Additionals modal waiting timeout");
            expect(yield $modal.$$(".list-group .react-spinner").get(1).isDisplayed()).toBe(true);
            for (let idx in informalGroups) {
                yield mocks[idx].resolve({
                    "id": informalGroups[idx],
                    "name": "test"
                });
            }
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.invisibilityOf($modal.$(".list-group .react-spinner")), 5000, "Additionals group title spinner didn't hide");
            expect(yield $modal.$$(".list-group .react-spinner").count()).toBe(0);
            expect(yield $modal.$$(".list-group").get(1).$(".list-group-item strong").getText()).toBe("test");
        }));
        it("can add additional fields", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield test_utils_1.lajiFormLocate("gatherings.0.units.0.identifications.0.det").isPresent()).toBe(false);
            expect(yield $additionalsButton.isDisplayed()).toBe(true);
            const $firstGroup = protractor_1.$$(".scope-field-modal-item").first();
            const getFieldCount = () => test_utils_1.lajiFormLocate("gatherings.0.units.0").$$("input").count();
            const fieldCount = yield getFieldCount();
            const $additionalItem = $firstGroup.all(protractor_1.by.css(".list-group-item")).get(1);
            expect(yield $modal.isDisplayed()).toBe(true);
            expect(yield $additionalItem.isDisplayed()).toBe(true);
            yield $additionalItem.click();
            yield $modal.$(".close").click();
            expect(yield $additionalItem.isPresent()).toBe(false);
            expect(yield yield getFieldCount()).toBe(fieldCount + 1);
        }));
        const $getLocationButtonFor = (gatheringIdx, unitIdx) => protractor_1.$(`#root_gatherings_${gatheringIdx}_units_${unitIdx}-location`);
        const $locationModal = protractor_1.$(".map-dialog");
        const $locationModalMap = $locationModal.$(".laji-map");
        it("has location button", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $getLocationButtonFor(0, 0).isDisplayed()).toBe(true);
        }));
        function clickLocationButtonAndAddLocation(gatheringIdx, unitIdx) {
            return __awaiter(this, void 0, void 0, function* () {
                yield $getLocationButtonFor(gatheringIdx, unitIdx).click();
                yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf($locationModal), 5000, "Map modal waiting timeout");
                expect(yield $locationModal.isDisplayed()).toBe(true);
                expect(yield $locationModalMap.isDisplayed()).toBe(true);
                yield protractor_1.browser.actions()
                    .mouseMove($locationModalMap.getWebElement(), { x: 100, y: 100 }).perform();
                yield protractor_1.browser.actions().click().perform();
            });
        }
        it("can add location", () => __awaiter(void 0, void 0, void 0, function* () {
            yield clickLocationButtonAndAddLocation(0, 0);
            expect(yield $gatheringsMap.$$(".vector-marker.leaflet-interactive").count()).toBe(2);
        }));
        it("unit map modal hides after adding location", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $locationModalMap.isPresent()).toBe(false);
        }));
        const $$gatheringMarkerPaths = $gatheringsMap.$$(".vector-marker path");
        it("gatherings map shows unit with different color", () => __awaiter(void 0, void 0, void 0, function* () {
            expect(yield $$gatheringMarkerPaths.count()).toBe(2);
            const firstFill = yield $$gatheringMarkerPaths.first().getAttribute("fill");
            const secondFill = yield $$gatheringMarkerPaths.last().getAttribute("fill");
            expect(yield firstFill).not.toBe(secondFill);
        }));
        it("hovering unit table row changes gathering map unit color", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $unitAdd.click();
            const unitFill = yield $$gatheringMarkerPaths.last().getAttribute("style");
            debugger;
            yield protractor_1.browser.actions()
                .mouseMove(test_utils_1.lajiFormLocate("gatherings.0.units.0").getWebElement())
                .perform();
            debugger;
            const unitFillAfterLocationHover = yield $$gatheringMarkerPaths.last().getAttribute("style");
            expect(unitFill).not.toBe(unitFillAfterLocationHover);
            yield test_utils_1.removeUnit(0, 1);
        }));
        const $locationPeeker = protractor_1.$("#root_gatherings_0_units_0-location-peeker");
        it("hovering location button displays location peeking map", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.browser.actions()
                .mouseMove($getLocationButtonFor(0, 0).getWebElement())
                .perform();
            expect(yield $locationPeeker.isDisplayed()).toBe(true);
        }));
        it("location peeker shows data", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.browser.actions()
                .mouseMove($getLocationButtonFor(0, 0).getWebElement())
                .perform();
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf($locationPeeker.$(".leaflet-container")), 5000, "Map peeker waiting timeout");
            const $$peekerPaths = protractor_1.$$("#root_gatherings_0_units_0-location-peeker .vector-marker path");
            expect(yield $$peekerPaths.count()).not.toBe(0);
            const firstFill = yield $$peekerPaths.first().getAttribute("fill");
            const secondFill = yield $$peekerPaths.last().getAttribute("fill");
            expect(yield firstFill).not.toBe(secondFill);
        }));
        it("can have location even if gathering doesn't have", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $gatheringAdd.click();
            yield clickLocationButtonAndAddLocation(1, 0);
            yield test_utils_1.waitUntilBlockingLoaderHides(6000);
            expect(yield $$gatheringMarkerPaths.count()).toBe(1);
            yield protractor_1.$("#root_gatherings_1-delete").click();
            yield protractor_1.$("#root_gatherings_1-delete-confirm-yes").click();
            if (!googleApiKey) {
                yield test_utils_1.waitUntilBlockingLoaderHides(6000);
            }
        }));
        it("choosing informal taxon group changes fields", () => __awaiter(void 0, void 0, void 0, function* () {
            yield protractor_1.$(".informal-taxon-group-chooser").click();
            const getFieldCount = () => test_utils_1.lajiFormLocate("gatherings.0.units.0").$$("input").count();
            const fieldCount = yield getFieldCount();
            const $birdButton = protractor_1.element(protractor_1.by.className("MVL.1")).element(protractor_1.by.xpath("..")).$$("button").last(); // eslint-disable-line protractor/no-by-xpath
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf($birdButton), 5000, "Bird button didn't show up");
            yield $birdButton.click();
            yield protractor_1.browser.wait(protractor_1.protractor.ExpectedConditions.visibilityOf(test_utils_1.lajiFormLocate("gatherings.0.units.0.twitched")), 4000, "Bird field didn't appear");
            expect(yield fieldCount).not.toBe(yield getFieldCount());
        }));
        it("adding image in background", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $unitAdd.click();
            const { resolve, remove } = yield form.mockImageUpload("gatherings.0.units.1");
            yield form.$locate("gatherings.0.units.0").$$("td").first().click();
            yield resolve();
            yield form.$locate("gatherings.0.units.1").$$("td").first().click();
            expect(yield form.getImageArrayField("gatherings.0.units.1").$$imgs.count()).toBe(1);
            yield remove();
            yield test_utils_1.removeUnit(0, 1);
        }));
        it("removing unit with image loading", () => __awaiter(void 0, void 0, void 0, function* () {
            yield $unitAdd.click();
            const { resolve, remove } = yield form.mockImageUpload("gatherings.0.units.1");
            yield form.$locate("gatherings.0.units.0").$$("td").first().click();
            yield test_utils_1.removeUnit(0, 1);
            yield resolve();
            yield remove();
            expect(yield form.failedJobs.$container.isPresent()).toBe(false);
        }));
    });
});
